/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * OPSBrowserInterpreter.java
 * 
 * Time-stamp: <2003-03-07 22:43:40 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: OPSBrowserInterpreter.java,v 1.16 2008/10/30 14:53:13 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen;

import laas.friends.cosmopen.interpreter.* ;
import laas.friends.cosmopen.commands.* ;
import laas.friends.cosmopen.graphAnalysis.CallGraphDotWriter ;
import laas.friends.cosmopen.graphAnalysis.CallGraphToCondensedGraphTransformer ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict2DotOStream ;
import laas.friends.cosmopen.interpreter.InterpreterException ;
import java.io.File ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.16 $
 * @since Version 0.0.2
 */
public class OPSBrowserInterpreter 
    extends    CommandInterpreter 
    implements OptionListener
{

    //    final static String VERSION="0.1.4 [under work]" ;
    final static String VERSION="Release $Name: release-1-3-rc1 $ $Revision: 1.16 $" ;

    protected void printLicenceInfo() {
	CommandInterpreter.out.println
	    ("\nCopyright (C) 2003-2004 LAAS-CNRS, (C) 2004-2005 Francois Taiani\n"+
	     "Author: Francois Taiani\n" +
	     "CosmOpen comes with ABSOLUTELY NO WARRANTY.\n" +
	     "This is free software, and you are welcome to redistribute it\n" +
	     "under certain conditions. See the included Licence for more \n"+
	     "details.\n") ;
    } // EndMethod printLicenceInfo

    public OPSBrowserInterpreter() {
	super("CosmOpen/OPSBrowser ("+VERSION+")" ) ;
	// exports command instances to the interpreter.
	
	Command clearall    = new ClearAllCommand  () ;
	Command toPs        = new ToPsCommand      () ;
	Command suppress    = new SuppressCommand  () ;
	Command remUsage    = new RemoveLinkCommand("remUse","usage"     ) ;
	Command remHeritage = new RemoveLinkCommand("remHin","superclass") ;

	GroupOfDocumentedItemS inputOutput = 
	    new GroupOfDocumentedItemS
	    ("inout",
	     "input / output operations with graphs",
	     "This group contains operations to load and save graphs to files using "+
	     "various formats.") ;

	addGroupToKnownOnes( inputOutput  ) ;

 	addItemToKnownOnesIntoGroup( clearall , inputOutput ) ;
	addItemToKnownOnesIntoGroup( toPs     , inputOutput ) ;
	addItemToKnownOnesIntoGroup( new LoadCommand    (), inputOutput ) ;
	addItemToKnownOnesIntoGroup( new ToDotCommand   (), inputOutput ) ;
	addItemToKnownOnesIntoGroup( new ToDotRawCommand(), inputOutput ) ;
	addItemToKnownOnesIntoGroup( new SaveCommand    (), inputOutput ) ;
 	addItemToKnownOnesIntoGroup( new Bind2PsCommand (), inputOutput ) ;
 	addItemToKnownOnesIntoGroup( new BindRawCommand (), inputOutput ) ;

	GroupOfDocumentedItemS manipulation = 
	    new GroupOfDocumentedItemS
	    ("transform",
	     "graph transformation",
	     "This group contains operations to create new graphs from existing ones.") ;

	addGroupToKnownOnes( manipulation ) ;

	addItemToKnownOnesIntoGroup( suppress   , manipulation ) ;
	addItemToKnownOnesIntoGroup( remUsage   , manipulation ) ;
	addItemToKnownOnesIntoGroup( remHeritage, manipulation ) ;
	addItemToKnownOnesIntoGroup( new ClearCommand        (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new AssignCommand       (), manipulation ) ;
 	addItemToKnownOnesIntoGroup( new PrintCommand        (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new RemoveCommand       (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new ExcludeCommand      (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new PutCommand          (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new AddCommand          (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new CleanCommand        (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new RemoveAloneCommand  (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new RemoveBefore        (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new RemoveAfter         (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new SliceCommand        (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new AbstractGraphAway   (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new AbstractPatternAway (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new AbstractAwaySelfCall(), manipulation ) ;
	addItemToKnownOnesIntoGroup( new LeapOver            (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new Rebind              (), manipulation ) ;
	addItemToKnownOnesIntoGroup( new Fuse                (), manipulation ) ;
 	GroupOfDocumentedItemS extension = 
	    new GroupOfDocumentedItemS
	    ("extend",
	     "graph extension",
	     "This group contains operations based on graph traversals and edge operations.") ;

	addGroupToKnownOnes( extension ) ;

	addItemToKnownOnesIntoGroup( new BackwardCommand   (), extension ) ;
	addItemToKnownOnesIntoGroup( new ForwardCommand    (), extension ) ;
	addItemToKnownOnesIntoGroup( new SpreadCommand     (), extension ) ;
	addItemToKnownOnesIntoGroup( new EnvlopCommand     (), extension ) ;
	addItemToKnownOnesIntoGroup( new BackNCommand      (), extension ) ;
	addItemToKnownOnesIntoGroup( new ForwNCommand      (), extension ) ;
	addItemToKnownOnesIntoGroup( new SpreadNCommand    (), extension ) ;

	// THESE COMMAND ARE ADDED TO THE "general" GROUP BY DEFAULT

	addCommandToKnownOnes( new InfoCommand    () ) ;
	addCommandToKnownOnes( new HistoryCommand () ) ;
 	addCommandToKnownOnes( new ForgetCommand  () ) ;
 	addCommandToKnownOnes( new CommentCommand () ) ;

	// ADDING OPTIONS THAT CAN BE SET AND UNSET

	addOptionToInterpreter("umllike",					      
			       "Switches graph formatting to \"UML-like\"") ;
	addOptionToInterpreter("longrange",
			       "Switches explicit range formatting") ;
	addOptionToInterpreter("shortname",
			       "Removes namespace in graph output") ;
	addOptionToInterpreter("leftright",
			       "Edges are drawn from left to right in dot") ;
	addOptionToInterpreter("nopage",
			       "Don't paginate dot results") ;	
	addOptionToInterpreter("landscape",
			       "Use landscape orientation") ;	
	setListenerForOption  (this,"umllike") ;
	setListenerForOption  (this,"longrange") ;
	setListenerForOption  (this,"shortname") ;
	setListenerForOption  (this,"leftright") ;
	setListenerForOption  (this,"nopage") ;
	setListenerForOption  (this,"landscape") ;

	switchOptionOn("umllike");

    } // EndConstructor OPSBrowserInterpreter

    private void handleCommandLineArgs(String[] argv) throws InterpreterException {

	for(int i=0 ; i < argv.length ; i++) {
	    if (argv[i].charAt(0) != '-') {
		// We load the argument as a graph file
		System.out.println("Loading "+argv[i]) ;
		interpretCommand("load "+argv[i]) ;
	    } else if (argv[i].equals("-f")) {
		i++ ;
		if (i >= argv.length) {
		    System.err.println("-f requires a file name argument.");
		    printUsageInfoAndQuit() ;
		}
		System.out.println("Executing "+argv[i]) ;
		interpretCommand("run "+argv[i]) ;
	    } else if (argv[i].equals("-h")) {
		printUsageInfoAndQuit() ;                    
	    } else {
		System.err.println(argv[i] + " is an unknown option.");
		printUsageInfoAndQuit() ;
	    }
	} // EndFor

    } // EndMethod handleCommandLineArgs

    private String defaultConfigFileName = "./opsbrowser_config" ;

    private void runDefaultConfigIfExist() throws InterpreterException {

	File configFile = new File(defaultConfigFileName);
	if (!configFile.exists()) return ;
	System.out.println("Executing default config file:") ;
	interpretCommand("run "+defaultConfigFileName) ;
	System.out.println();
	
    } // EndMethod runDefaultConfigIfExist

    // **************************************************************
    // MAIN

    /**
     * Lauches an instance of {@link OPSBrowserInterpreter}
     */
    static public void main(String argv[]) {

	OPSBrowserInterpreter theInterpreter = new OPSBrowserInterpreter() ;

	try {
	    theInterpreter.printLicenceInfo() ;
	    theInterpreter.runDefaultConfigIfExist();
	    theInterpreter.handleCommandLineArgs(argv);
	    theInterpreter.launchInterpreter() ;
	}
	catch (InterpreterException e) {
	    System.err.println
                ("INTERNAL ERROR: "+e.getMessage()) ;
            System.exit(1) ;
	} // EndCatch

    } // EndMain

    public static void printUsageInfoAndQuit() {
        CommandInterpreter.out.println();
        CommandInterpreter.out.println
            ("Usage : OPSBrowser -h");
        CommandInterpreter.out.println
            ("        OPSBrowser [-f <commandFile>] [graphFile ..]" );
        CommandInterpreter.out.println();
        System.exit(0) ;
    } // EndMain printUsageInfo

    public void optionHasNewValue(String anOption, String aValue) {
	if (anOption.equals("umllike")) {
	    CallGraphDotWriter.setUmlLikeFormat((aValue!=null)) ;
// 	    DirectedGraphWithDict2DotOStream
// 		.setUseShortName ((aValue!=null) || this.optionIsSet("shortname")) ;
	}
	if (anOption.equals("longrange"))
	    CallGraphToCondensedGraphTransformer.setExplicitRange((aValue!=null)) ;
	if (anOption.equals("shortname"))
	    DirectedGraphWithDict2DotOStream
		.setUseShortName ((aValue!=null) || this.optionIsSet("umllike")) ;
	if (anOption.equals("leftright")) 
	    DirectedGraphWithDict2DotOStream.setLeftRight((aValue!=null));
	if (anOption.equals("nopage"))
	    DirectedGraphWithDict2DotOStream.setSkipPagination((aValue!=null));
	if (anOption.equals("landscape"))
	    DirectedGraphWithDict2DotOStream.setLandscape((aValue!=null));

    } // EndMethod optionHasNewValue

} // EndClass OPSBrowserInterpreter
