/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractBackForwCommand.java
 * 
 * Time-stamp: <02/08/08 16:38:44 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AbstractBackForwCommand.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;

/**
 * Abstract class for all backward and forward command, 
 * either unlimited
 * in their recursion or limited.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public abstract class AbstractBackForwCommand 
    extends CommandWithOptionalContext {

    private int _posVarToModif = 1 ;
    /**
     * determines wether the operation is a backward or forward one. 
     * @see #BACKWARD
     * @see #FORWARD 
     */
    private int _direction ;
    public static int BACKWARD = -1 ;
    public static int FORWARD  =  1 ;
    public static int SPREAD   =  0 ;

    /**
     * @param direction is either {@link #BACKWARD } or {@link #FORWARD   }
     */
    public AbstractBackForwCommand( String commandName, 
				    int    direction ) {
	super(commandName) ;
	_direction     = direction ;
    } // EndConstructor AbstractBackForwCommand

    /**
     * @param direction is either {@link #BACKWARD } or {@link #FORWARD   }
     */
    public AbstractBackForwCommand(String commandName,
				   int    minArgs,
				   int    maxArgs, 
				   int    direction  ) 
    {
	super(commandName,minArgs,maxArgs) ;
	_direction     = direction ;
	_posVarToModif = maxArgs-1 ;
    } // EndConstructor AbstractBackForwCommand

    protected boolean handleCallInternal(String[] args, int aDepthLimit )
    throws Exception {

	if ( CheckIsBaseGraphAndErrorIfYes(args[_posVarToModif])) 
	    return false ;

	boolean executionOk = false ;

	try {
	    DirectedGraphWithDict contextGraph ;
	    String                contextName = getContextName(args);
	    contextGraph = GraphVariable.GetGraphForName(contextName) ;
	    DirectedGraphWithDict graphUnderWork 
		= GraphVariable.GetGraphForName(args[_posVarToModif]) ;

	    if (_direction==BACKWARD) 
		graphUnderWork = 
		    contextGraph.
		    extractFixBackwardFromGraph(graphUnderWork,aDepthLimit) ;
	    if (_direction==FORWARD) 
		graphUnderWork = 
		    contextGraph.
		    extractFixForwardFromGraph(graphUnderWork,aDepthLimit) ;

	    if (_direction==SPREAD) {
		DirectedGraphWithDict backwardClosure = 
		    contextGraph.
		    extractFixBackwardFromGraph(graphUnderWork,aDepthLimit) ;
		DirectedGraphWithDict result = 
		    contextGraph.
		    extractFixForwardFromGraph(graphUnderWork,aDepthLimit) ;
		result.addGraph(backwardClosure) ;
		graphUnderWork = result ;
	    } // EndIf _direction == SPREAD

	    GraphVariable.SetVariableToValue
		(args[_posVarToModif],graphUnderWork) ;
	    GraphVariable.NotifyChangeOnVariable(args[_posVarToModif],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[_posVarToModif],contextName},
		  new String[] {args[_posVarToModif]}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndClass AbstractBackForwCommand
