/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractGraphAway.java
 * 
 * Time-stamp: <2003-03-07 23:44:47 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AbstractGraphAway.java,v 1.3 2003/06/13 16:38:22 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.* ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.1.2
 */
public class AbstractGraphAway extends AbstractAbstractAwayCmd {

    public AbstractGraphAway() {
	super("abstract") ;
	addAlias("ab");
	addAlias("%%");
// 	setShortOutput(true) ;
	this.setShortDescription
	    ( "abstracts away the nodes present in one graph from another graph.") ;
	this.setLongDescription
	    ( "'abstract <graph1> <graph2>' abstracts away the nodes present "+
	      "in <graph1> from <graph2>. Abstracting away means nodes are removed "+
	      "but their childs are re-connected to their unique parent.\n"+
	      "This is specific to call graphs and won't work on other graphs.") ;

    } // EndConstructor AbstractGraphAway

    protected void notifyChangeToHistory(String[] args) {
	GraphVariable.NotifyChangeOnVariable(args[2],args) ;
	GraphVariable.PutHistoryArgsInputOutput
	    ( args, 
	      new String[] {args[2],args[1]},
	      new String[] {args[2]}
	      );
    } // EndMethod notifyChangeToHistory

    protected Collection getNodeSToRemoveFromCondition(String condition) 

	throws IllegalArgumentException
    
    {
	return	GraphVariable.GetGraphForName(condition).cloneVertices() ;
    } // EndMethod getNodeSToRemoveFromCondition

    public boolean handleCall(String[] args) throws Exception {

	if ( ! doNecessaryVariableChecks(args,1,2) ) return false ;
	return super.handleCall(args) ;

    } // EndMethod handleCall

} // EndClass AbstractGraphAway
