/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * CleanCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:45 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: CleanCommand.java,v 1.3 2003/10/13 07:25:42 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands ;

import laas.friends.cosmopen.graphAnalysis.* ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.4
 */
public class CleanCommand extends CommandWithVariableArgs {

    public CleanCommand() {
	super("clean",1,2) ;
	this.setShortDescription
	    ( "removes the 'weakly link' nodes from a graph.") ;
	this.setLongDescription
	    ( "'clean [<aNode>] <aVariableName>' when no first node argument "+
	      "is given removes all nodes which either stand alone or "+
	      "only have one neighbor. When <aNode> is given, only the "+
	      "neighbors around that node are concerned.") ;
    } // EndConstructor CleanCommand

    public boolean handleCall(String[] args) throws Exception {
	int graphArg = args.length-1 ; // To cope with the optional first arg.

	if ( !doNecessaryVariableChecks(args,graphArg,graphArg)  ) 
	    return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(args[graphArg]) ) return false ;

	boolean executionOk = false ;
	try {
	    DirectedGraphWithDict graphUnderFire 
		= GraphVariable.GetGraphForName(args[graphArg]) ;
	    if (args.length==3) { // There is a 'node' argument
		NamedVertex concernedNode = 
		    graphUnderFire.getNodeWithName(args[1]);
		
		if (concernedNode==null) {
		    println("No Nodes '"+args[1]+
				       "' in graph '"+args[graphArg]+"'.") ;
		    return false ;
		} // concernedNode==null

		graphUnderFire.cleanAroundVertex(concernedNode) ;
	    } // EndIf There is a node argument
	    else {
		java.util.Iterator iterator 
		    = graphUnderFire.cloneVertices().iterator() ;

		// ** LOOP THROUGH ALL NODES

		boolean somethingRemoved = false ;
		while (iterator.hasNext() ) {

		    NamedVertex currentClassD  = (NamedVertex)iterator.next() ;
		    if (graphUnderFire.doesContainVertex(currentClassD))
			if (graphUnderFire.getDegree(currentClassD)<=1) {
			    println   (currentClassD.getName()) ;
			    graphUnderFire.remove(currentClassD) ;
			    somethingRemoved = true ;
			} // EndIf degree(currentClassD)==0
		
		} // EndWhile LOOP THROUGH ALL NODES

		if (!somethingRemoved) {
		    println("No Matching Nodes in graph '"+
				       args[graphArg]+"'.") ;
		    return false ;
		} // No nodes were removed

		println("   were removed from "+args[graphArg]) ;

	    } // EndElse There is no node arg. Whole graph is concerned

	    GraphVariable.NotifyChangeOnVariable(args[graphArg],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[graphArg]},
		  new String[] {args[graphArg]}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndClass CleanCommand
