/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * ClearCommand.java
 * 
 * Time-stamp: <02/08/13 22:23:29 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: ClearCommand.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.interpreter.Command ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDictImpl ;

/**
 * A command to clear the content of a variable.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.3
 */
public class ClearCommand extends Command {

    public ClearCommand() {
	super("clear",1,-1) ;
	this.setShortDescription
	    ( "removes all nodes from a graph variable.") ;
	this.setLongDescription
	    ( "'clear <var1> [<var2>..<varN>]' removes all the nodes from "+
	      "the variables "+
	      "whose names are passed as parameter. It is not possible "+
	      "to clear the base graph with this command. Use 'clearall' "+
	      "instead.") ;
    } // EndConstructor ClearCommand

    public boolean handleCall(String[] args) {
	if ( ! thereIsRightNumberOfArgs(args) ) return false ;

	java.util.List modifiedVar = new java.util.Vector() ;

	for (int i=1 ; i < args.length ; i++) {

	    if ( ! GraphVariable.VariableWithNameIsKnown(args[i]) ) {
		println
		    ( "Variable with name '"+args[i]+"' is not defined.") ; 
		return false ;
	    } // EndIf VariableWithNameIsKnown
	    else 
		if (args[i].equals(GraphVariable.NAME_OF_BASEALIAS)) {
		    println
			("ERROR: It is not possible "+
			 "to clear the base graph with this command. "+
			 "Use 'clearall' instead.") ;
		    return false ;
		} // EndIf passed variable is base graph
		else {
		    GraphVariable.SetVariableToValue
			(args[i], GraphVariable.NewEmptyGraphSameTypeAsBase() ) ;
		    GraphVariable.NotifyChangeOnVariable
			(args[i], new String[] {args[0],args[i]});
		    println("Variable '"+args[i]+"' cleared.") ;
		    modifiedVar.add(args[i]) ;
		} // EndElse everythin ok, we can clear
	} // EndFor i

	GraphVariable.PutHistoryArgsInputOutput
	    ( args, 
	      new String[] {},
	      (String[])modifiedVar.toArray(new String[]{})
	      );

	return true ;
    } // EndMethod handleCall

} // EndClass ClearCommand
