/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * EnvlopCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:32 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: EnvlopCommand.java,v 1.2 2003/04/07 18:10:35 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.3
 */
public class EnvlopCommand extends CommandWithOptionalContext {

    public EnvlopCommand() {
	super("envlop") ;
	this.setShortDescription
	    ( "computes the edge envelop of a graph.") ;
	this.setLongDescription
	    ( "'envlop <aVariableName>' completes "+
	      "the graph passed as argument with "+
	      "the edges found in the base graph which go from and to nodes "+
	      "of the passed graph. (A kind of \"envelop\" operation.)") ;
    } // EndConstructor EnvlopCommand

    public boolean handleCall(String[] args) throws Exception {
	if ( ! passedVariableDoesExist(args) ) return true ;

	boolean executionOk = false ;

	try {
	    DirectedGraphWithDict contextGraph ;
	    String                contextName = getContextName(args);
	    contextGraph = GraphVariable.GetGraphForName(contextName) ;
	    DirectedGraphWithDict graphUnderWork 
		= GraphVariable.GetGraphForName(args[1]) ;
	    graphUnderWork.doEdgeCompletionInContext(contextGraph) ;
	    GraphVariable.NotifyChangeOnVariable(args[1],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[1],contextName},
		  new String[] {args[1]}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndClass EnvlopCommand
