/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * ExcludeCommand.java
 * 
 * Time-stamp: <2003-03-11 11:20:25 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: ExcludeCommand.java,v 1.3 2004/10/04 21:47:42 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;
import laas.friends.cosmopen.graphAnalysis.NamedVertex ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public class ExcludeCommand extends CommandWithVariableArgs {

    public ExcludeCommand() {
	super("exclude",2,2) ;
	addAlias("e");
	addAlias("/");
	this.setShortDescription
	    ( "excludes the nodes of a given graph from a second.") ;
	this.setLongDescription
	    ( "'exclude <firstGraph> <secondGraph>' removes the nodes "+
	      "which are in the first graph from the second. You can "+
	      "use the alias 'base' to remove nodes directly from the base graph.") ;
    } // EndConstructor ExcludeCommand

    public boolean handleCall(String[] args) throws Exception {

	if ( ! doNecessaryVariableChecks(args,1,2) ) return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(args[2])) return false ;

	boolean executionOk = false ;

	try {
	    java.util.Collection nodesToExcude 
		= GraphVariable.GetGraphForName(args[1]).cloneVertices() ;
	    DirectedGraphWithDict graphToBeModified 
		= GraphVariable.GetGraphForName(args[2]) ;

	    if (nodesToExcude.isEmpty())
		println("'"+args[1]+"' contains no nodes.") ;
	    else {

		// *** LOOP THROUGH NODES TO BE EXCLUDED

		java.util.Iterator iter = nodesToExcude.iterator() ;
                
                graphToBeModified.removeVertexWithIterator(iter) ;

		println(args[1]+" was removed from "+args[2]+".") ;
		GraphVariable.NotifyChangeOnVariable(args[2],args) ;
		GraphVariable.PutHistoryArgsInputOutput
		    ( args, 
		      new String[] {args[2],args[1]},
		      new String[] {args[2]}
		      );
		executionOk = true ;
	    } // EndElse there are matching nodes in passed graph
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch

	return executionOk ;

    } // EndMethod handleCall
    
} // EndClass ExcludeCommand
