/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * Fuse.java
 * 
 * Time-stamp: <2003-03-11 13:49:50 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: Fuse.java,v 1.1 2008/10/30 14:32:19 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.* ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision $
 */
public class Fuse extends AbstractLeapAndRebind {

    public Fuse() {
 	super("fuse",3,3) ;
	addAlias("fu");
	addAlias("[]");
	this.setShortDescription
	    ( "fuse children to parents based on temporal ordering") ;
	this.setLongDescription
	    ( 

	     "'fuse <startPattern> <fusePattern> <graph>' searches for all nodes S "+
	     "that match <startPattern> and all nodes N that match "+
	     "<fusePattern>. It then matches nodes in both set "+
	     "pair-wise according to the order in which they appear "+
	     "(the first node in S with the first node in N, etc) and "+
	     "changes the parent of each node N to the matching node in "+
	     "S. This corresponds to a kind of temporal jump from P to "+
	     "N. The sequence of nodes in S is \"fused\" to the "+
	     "sequence of nodes in N. The two sequence must have the "+
	     "same number of nodes for the operation to succeed, and "+
	     "each new parent should have an earlier timestamp than "+
	     "its newly associated child. If these conditions can not be "+
	     "met an error occurs and no change is made to the graph. "+
	     "Like leapOver and rebind, fuse can be useful to take into "+
	     "account dependencies that are not reflected in the trace "+
	     "trees in a CallGraph (pipes, ...etc).\n This is specific "+
	     "to call graphs and won't work on other graphs."

	      ) ;
    } // EndConstructor Fuse

    public void doTransform(CallGraph sourceGraph, 
			    Set       startNodes, 
			    Set       endNodes)
	throws Exception
    {
	sourceGraph.fuse(startNodes,endNodes);
    } // EndMethod doTransform
    

} // EndClass Fuse
