/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * LeapOver.java
 * 
 * Time-stamp: <2003-03-11 13:49:50 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: LeapOver.java,v 1.4 2007/09/18 18:23:42 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen.commands;

import laas.friends.cosmopen.graphAnalysis.* ;
import java.util.* ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision $
 */
public class LeapOver extends AbstractLeapAndRebind {

    public LeapOver() {
 	super("leapOver",3,3) ;
	addAlias("lpo");
	addAlias("&");
	this.setShortDescription
	    ( "leap over calls based on temporal ordering") ;
	this.setLongDescription
	    ( "'leapOver <startPattern> <leapPattern> <graph>' searches for all nodes N that " +
	      "match <leapPattern>. For each of these nodes, it searches the most recent "+
	      "node P matching <startPattern> that has occurred *before* Node N. It then "+
	      "moves all outgoing links from N to P. This corresponds to a kind of "+
	      "temporal jump from P to the successors of N. N is \"leapt over\". "+
	      "This can be useful to take into account dependencies that are not reflected "+
	      "in the trace trees in a CallGraph (pipes, ...etc).\n"+
	      "This is specific to call graphs and won't work on other graphs.") ;
    } // EndConstructor LeapOver

    public void doTransform(CallGraph sourceGraph, 
			    Set       startNodes, 
			    Set       endNodes)
	throws Exception
    {
	sourceGraph.leapOverNodes(startNodes,endNodes);
    } // EndMethod doTransform
    

} // EndClass LeapOver
