/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * OutputCommand.java
 * 
 * Time-stamp: <2002-08-26 14:10:12 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: OutputCommand.java,v 1.5 2005/04/13 09:42:10 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import java.util.Date ;
import java.text.SimpleDateFormat ;
import  laas.friends.cosmopen.utils.MiscUtils ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.5 $
 * @since Version 0.0.3
 */
public abstract class OutputCommand extends CommandWithVariableArgs {

    /** Format the current time. */
    private static SimpleDateFormat formatter = 
	new SimpleDateFormat ("dd-MM-yyyy 'at' kk:mm");

    public OutputCommand(String aName) {
	super(aName,1,1) ;
   } // EndConstructor OutputCommand

    public OutputCommand(String aName,int max,int min) {
	super(aName,max,min) ;
   } // EndConstructor OutputCommand

    /**
     * return a description of the global history so far in a dot
     * format, which can then be passed to {@link
     * laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict2DotOStream#setComment(String)
     * DirectedGraphWithDict2DotOStream.setComment(..)}
     */
    public String getCommentForArgsFileAndVar( String[] args,
					       String   aFileName, 
					       String   aPrintedVar ) 
    {
	java.util.List historyToPrint =
	    GraphVariable.GetHistoryForVariable(aPrintedVar) ;

	StringBuffer comment = new StringBuffer() ;
	int lineCount = 0 ;
	comment.append("{") ;
	comment.append("OPSBrowser\\n") ;
	lineCount++ ;
	String dateString = formatter.format(new Date());
	comment.append("|{") ;
	comment.append( dateString+ "\\n" ) ;
	lineCount++ ;
	comment.append( aFileName + "\\nHISTORY: \\l" ) ;
	lineCount+=2 ;

	java.util.Iterator iterHist = historyToPrint.iterator() ;
	while(iterHist.hasNext()) {
	    comment.append("  "+iterHist.next()+"\\l");
	    lineCount++ ;
	    if (lineCount>10) {
		lineCount = 0 ;
		comment.append("|") ;
	    } // EndIf (lineCount>10)
	} // EndWhile iterHist.hasNext()

	// Appending file production command itself.
	comment.append("  ") ;

	for(int i=0; i<args.length;i++) {
	    comment.append(args[i]);
	    if (i<args.length-1) 
		comment.append(" ");
	    else
		comment.append("\\l");
	} // EndFor i Loops through arguments

	comment.append("}") ;
	comment.append("}") ;
	String theFinalResult = comment.toString() ;

	// Dot needs '<' and '>' to be escaped.

	theFinalResult = MiscUtils.ReplaceCharWithString(theFinalResult,'<',"\\<") ;
	theFinalResult = MiscUtils.ReplaceCharWithString(theFinalResult,'>',"\\>") ;

	return  theFinalResult ;

    } // EndMethod getComment

} // EndClass OutputCommand
