/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * PutCommand.java
 * 
 * Time-stamp: <2002-08-16 16:50:12 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: PutCommand.java,v 1.3 2003/10/13 07:25:42 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDict ;
import laas.friends.cosmopen.graphAnalysis.DirectedGraphWithDictImpl ;
import laas.friends.cosmopen.graphAnalysis.NamedVertex ;

import java.util.Collection ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.3
 */
public class PutCommand extends AbstractPutCommand {

    public PutCommand() {
	super("put") ;
	addAlias("p");
	addAlias("+");
	this.setShortDescription
	    ( "puts nodes from one graph to another w.r.t. a pattern.") ;
	this.setLongDescription
	    ( "'put <aPattern> [<sourceGraph>] <targetGraph>' retrieves the nodes "+
	      "of a source graph whose names match a given pattern and puts them in "+
	      "a target graph.\n"+
	      "<aPattern> is a limited wildcard expression. I.e. stars "+
	      "('*') are allowed either at the beginning or at the end of the "+
	      "pattern but not in the middle.\n"+
	      "<sourceGraph> is the name of a graph variable. This variable must "+
	      "already have been declared and initialized. \n"+
	      "<sourceGraph> is the name of a graph variable. If this "+
	      "variable already exists, the found nodes are added to "+
	      "the already existing graph. Otherwise a new variable with "+
	      "this name is instantiated and receives the result of the name "+
	      "name matching.\n"+
	      "When <sourceGraph> is not provided the base graph is used instead.") ;
    } // EndConstructor PutCommand

   protected Collection getNodeSToAddFromCondition
       ( String condition, DirectedGraphWithDict sourceGraph) {
       return sourceGraph.getNodeSWithRegExp(condition) ;
   }

} // EndClass PutCommand
