/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * RemoveAloneCommand.java
 * 
 * Time-stamp: <02/08/15 20:59:25 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: RemoveAloneCommand.java,v 1.2 2003/04/07 18:10:36 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.* ;


/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public class RemoveAloneCommand extends CommandWithVariableArgs {

    public RemoveAloneCommand() {
	super("remAlone") ;
	addAlias("ral");
	this.setShortDescription
	    ( "removes the standalone nodes of a graph") ;
	this.setLongDescription
	    ( "'remAlone <aVariableName>' removes the nodes that have"+
	      "either incoming nor outgoing edges.\n" ) ;
    } // EndConstructor RemoveAloneCommand

    public boolean handleCall(String[] args) throws Exception {
	if ( ! doNecessaryVariableChecks(args,1,1)  ) return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(args[1]) ) return false ;

	boolean executionOk = false ;
	try {
	    DirectedGraphWithDict graphUnderFire 
		= GraphVariable.GetGraphForName(args[1]) ;

	    java.util.Iterator iterator 
		= graphUnderFire.cloneVertices().iterator() ;

	    // ** LOOP THROUGH ALL NODES

	    boolean somethingRemoved = false ;
	    while (iterator.hasNext() ) {
		NamedVertex currentClassD  = (NamedVertex)iterator.next() ;
		if (graphUnderFire.getDegree(currentClassD)==0) {
		    println   (currentClassD.getName()) ;
		    graphUnderFire.remove(currentClassD) ;
		    somethingRemoved = true ;
		} // EndIf degree(currentClassD)==0
		
	    } // EndWhile LOOP THROUGH ALL NODES

	    if (!somethingRemoved) {
		println("No Matching Nodes in graph '"+args[1]+"'.") ;
		return false ;
	    } // No nodes were removed

	    println("   were removed from "+args[1]) ;
	    GraphVariable.NotifyChangeOnVariable(args[1],args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[1]},
		  new String[] {args[1]}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall

} // EndClass RemoveAloneCommand
