/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * RemoveLinkCommand.java
 * 
 * Time-stamp: <02/08/08 16:44:52 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: RemoveLinkCommand.java,v 1.2 2003/04/07 18:10:36 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;
import laas.friends.cosmopen.graphAnalysis.* ;

/**
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public class RemoveLinkCommand extends CommandWithVariableArgs {

    private String _kindOfEdgesRemoved ;

    public RemoveLinkCommand(String aName, String aKind) {
	super(aName,1,2) ;
	_kindOfEdgesRemoved = aKind ;
	this.setShortDescription
	    ( "removes the link of kind '"+aKind+"' from a graph.") ;
	this.setLongDescription
	    ( "'remove [<var1>] <var2>' removes the links of kind '"+aKind+"' "+
	      "from the graph <var2>. "+
	      "In case <var1> is specified, only the links which are present " +
	      "in <var2> *and* in <var1> are concerned.") ;
    } // EndConstructor RemoveLinkCommand

    public boolean handleCall(String[] args) throws Exception {

	String varToBeModified = args[args.length-1];

	if ( ! doNecessaryVariableChecks(args,1,2)  ) return false ;
	if ( CheckIsBaseGraphAndErrorIfYes(varToBeModified) ) return false ;

	boolean executionOk = false ;
	try {
	    DirectedGraphWithDict graphUnderFire 
		= GraphVariable.GetGraphForName(varToBeModified) ;
	    DirectedGraphWithDict filterGraph
		= GraphVariable.GetGraphForName(args[1]) ;

	    // Here is there is only one variable, filterGraph and
	    // graphUnderFire are the same graph.

	    java.util.Iterator edgeIterator 
		= filterGraph.getEdgeIteratorKind(_kindOfEdgesRemoved) ;

	    if (!edgeIterator.hasNext()) {
		println("No Matching Links in graph '"+args[1]+"'.") ;
		return executionOk ;
	    } // !edgeIterator.hasNext()

	    graphUnderFire.removeEdgesWithIterator(edgeIterator) ;
	    GraphVariable.NotifyChangeOnVariable(varToBeModified,args) ;
	    GraphVariable.PutHistoryArgsInputOutput
		( args, 
		  new String[] {args[1],varToBeModified},
		  new String[] {varToBeModified}
		  );
	    executionOk = true ;
	} 
	catch (DirectedGraphWithDict.NotALimitedWilcardExpression except) {
	    println( "ERROR WITH EXPRESSION: " + except ) ;
	} // EndCatch
	return executionOk ;
    } // EndMethod handleCall
    

} // EndClass RemoveLinkCommand
