/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestAssignCommand.java
 * 
 * Time-stamp: <2003-03-11 11:30:46 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestAssignCommand.java,v 1.4 2004/09/09 15:31:00 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.commands  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.interpreter.* ;

/**
 * Test for {@link AssignCommand}.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.4 $
 * @since Version 0.0.4
 */
public class TestAssignCommand
extends AbstractCommandInterpreterTest {

    public TestAssignCommand(String name) {
	super(name) ;
    } // EndConstructor TestAssignCommand

    public void testGraphAreDistinctAfterCopy() 
	throws java.lang.Exception
    {
	DirectedGraphWithDict base = testeeCreator.createGraphTestee() ;
	GraphVariable.SetBaseGraph(base) ;

	// A new variable has an empty history

	DirectedGraphWithDict original = 
	    GraphVariable.ProvideGraphValueForName("original") ;
	java.util.Collection nodesToAdd = 
	    base.getNodeSWithRegExp("*") ;
	
	original.addNamedVertexS             ( nodesToAdd  ) ;
	original.doEdgeCompletionInContext ( base ) ;

	assertEquals( "Base and Original Should be equal", base, original);

	Command assign = new AssignCommand() ;
	String[] args ={"assign","original","new"} ;
	assign.handleCall(args);
	DirectedGraphWithDict newG =
	    GraphVariable.GetGraphForName("new") ;

	assertEquals( "Original and New Should be equal", original, newG );

	NamedVertex aNamedVertex 
	    = newG.getNodeWithName("BArbreCorde") ;
	newG.remove(aNamedVertex) ;

// 	println("Original = \n\n"+original+"\n\n"+
// 			   "NewG     = \n\n"+newG    +"\n\n") ;

	assertTrue("After Assign, 2 graphs should be independent.",
	       !original.equals(newG) ) ;

    } // EndMethod testGraphAreDistinctAfterCopy

    private laas.friends.cosmopen.graphAnalysis.test.GraphTesteeCreator 
	testeeCreator = new laas.friends.cosmopen.graphAnalysis.test.GraphTesteeCreator() ;
    
 } // EndClass TestAssignCommand
