/**
 * AbstractGuineaPig.java
 * 
 * Time-stamp: <2004-10-06 13:56:11 ftaiani>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: AbstractGuineaPig.java,v 1.1 2004/10/06 20:28:26 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.extractor.test ;

/**
 * This is the root class of the fake classes we use for
 * testing in {@link TestBreakpointTraceExtractor}.
 * @author Francois Taiani   <f.taiani@computer.org>
 */
abstract class AbstractGuineaPig {
    public abstract void myStartIsHere() ;
} // EndClass

class PlainGuineaPig extends AbstractGuineaPig {

    public String someResult = "" ;

    public void myStartIsHere() {
        method1();
    } // EndMethod myStartIsHere

    public void method1() {
        System.err.println("PlainGuineaPig.method1");
        method2("Hello") ;
    }
    
    public int method2(String blah) {
        System.err.println("PlainGuineaPig.method2");
        method3a(blah) ;
        method3b(blah + blah) ;
        return 0 ;
    }

    public void method3a(String blah) {
        System.err.println("PlainGuineaPig.method3a");
        someResult += blah ;
        method4() ;
    }

    public void method3b(String blah) {
        System.err.println("PlainGuineaPig.method3b");
        someResult += blah ;
    }

    public void method4() {
        System.err.println("PlainGuineaPig.method4");
        someResult += "blah4" ;
    }

} // EndClass PlainGuineaPig

class SecondPlainGuineaPig extends PlainGuineaPig {

    public int method2(String blah) {
        System.err.println("SecondPlainGuineaPig.method2");
        method3a(blah) ;
        method3b(blah + blah) ;
        return 0 ;
    }

    public void method3a(String blah) {
        System.err.println("SecondPlainGuineaPig.method3a");
        someResult += blah ;
        method4() ;
    }

} // EndClass SecondPlainGuineaPig

class ThreadedGuineaPig extends AbstractGuineaPig {

    public String someResult = "" ;
    public Thread theThreadICreated ;

    public void myStartIsHere() {
        method1();

        try {
            theThreadICreated.join();
	} catch (InterruptedException exc) {
	    // No reason to happen.
	}

    } // EndMethod myStartIsHere

    public void method1() {
        System.err.println("ThreadedGuineaPig.method1");
        method2("Hello") ;
    }
    
    public int method2(String blah) {
        System.err.println("ThreadedGuineaPig.method2");
        theThreadICreated = new ThreadThatIsCreatedByGuineaPig() ;
        theThreadICreated.start() ;
        method3b(blah + blah) ;
        return 0 ;
    }

    public void method3b(String blah) {
        System.err.println("ThreadedGuineaPig.method3b");
        someResult += blah ;
    }

} // EndClass

class ThreadThatIsCreatedByGuineaPig extends Thread {

    public String someResult = "" ;

    public void method3a(String blah) {
        System.err.println("ThreadThatIsCreatedByGuineaPig.method3a");
        someResult += blah ;
        method4() ;
    }

    public void method4() {
        System.err.println("ThreadThatIsCreatedByGuineaPig.method4");
        someResult += "blah4" ;
    }

    public void run() {
        method3a("Boom") ;
    }

} // EndClass ThreadThatIsCreatedByGuineaPig

class TestingOptionPassingClass extends AbstractGuineaPig {

    public String passedProperty = null ;

    public void myStartIsHere() {
        passedProperty = System.getProperty("somePropertyName");
        if (passedProperty==null) return ;
        if (passedProperty.equals("foo"))
            method1();
    } // EndMethod myStartIsHere

    public void method1() {
        System.err.println("ThreadedGuineaPig.method1");
    }
    
} // EndClass

class AdaptiveGuineaPig extends AbstractGuineaPig {

    public void myStartIsHere() {
        method1();
        method2("blah");
        method1();
    } // EndMethod myStartIsHere

    public void method1() {
        System.err.println("AdaptiveGuineaPig.method1");
    }
    
    public int method2(String blah) {
        System.err.println("AdaptiveGuineaPig.method2");
        return 0 ;
    }

} // EndClass AdaptiveGuineaPig
