/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * AbstractXMLReader.java
 * 
 * Time-stamp: <2002-08-27 13:04:41 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: AbstractXMLReader.java,v 1.6 2005/10/27 12:18:13 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import javax.xml.parsers.SAXParserFactory ;
import javax.xml.parsers.SAXParser ;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.ParserAdapter ;
import org.xml.sax.helpers.DefaultHandler ;

import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * Gathers some common functions of XML readers.
 * @version $Name: release-1-3-rc1 $ $Revision: 1.6 $
 * @since Version 0.1.1
 *
 */
public abstract class AbstractXMLReader 
    extends DefaultHandler 
    implements ProgressReporting
{
    private ProgressListener _myProgressListener ;

    public void setProgressListener(ProgressListener aProgressListener) {
	_myProgressListener = aProgressListener ;
    } // EndMethod setProgressListener

    protected void startListener(String msg) {
	if (_myProgressListener!=null) 
	    _myProgressListener.start(msg) ;
    } // EndMethod startListener

    protected void stopListener() {
	if (_myProgressListener!=null) 
	    _myProgressListener.end();
    } // EndMethod stopListener

    protected void tick() {
	if (_myProgressListener!=null) _myProgressListener.tick() ;
    } // EndMethod tick

    private ParserAdapter   internAdaptater ;

    protected void parse(java.io.InputStream anXMLSource) 
	throws 
	javax.xml.parsers.ParserConfigurationException
	,org.xml.sax.SAXException
	,java.io.IOException
    {
        SAXParserFactory tempFactory   = SAXParserFactory.newInstance();
        SAXParser        tempParser    = tempFactory.newSAXParser();
        internAdaptater = 
	    new ParserAdapter(tempParser.getParser());

	internAdaptater.setContentHandler(this);

	internAdaptater.parse(new org.xml.sax.InputSource(anXMLSource)) ;
    } // EndMethod parse

    /**
     * After {@link #parse(java.io.InputStream)} has been called,
     * the <code>AbstractXMLReader</code> is registered as the default
     * <code>content handler</code> for the XML document. This can be changed
     * during the parsing with this method.
     */
    protected void switchToNewXMLReader(AbstractXMLReader newDelegate) {
	newDelegate.initGraphConstruction() ;
	internAdaptater.setContentHandler(newDelegate);
	newDelegate.setProgressListener(_myProgressListener ) ;
    } // EndMethod switchToNewXMLReader

    /**
     * constructs a {@link DirectedGraphWithDict} from an XML stream. This
     * method first calls {@link #initGraphConstruction()}, then
     * {@link #parse(java.io.InputStream)}, and then returns the result of
     * {@link #endGraphConstructionAndReturnResult()}. {@link #initGraphConstruction()}
     * and {@link #endGraphConstructionAndReturnResult()} do nothing in
     * {@link AbstractXMLReader}, and must be implemented in subclasses.
     */
    public DirectedGraphWithDict readGraphFromXML  (java.io.InputStream anXMLSource)
	throws 
	javax.xml.parsers.ParserConfigurationException
	,org.xml.sax.SAXException
	,java.io.IOException
    {
	initGraphConstruction() ;
	parse(anXMLSource);
	DirectedGraphWithDict result 
	    = getCurrentXMLReader().endGraphConstructionAndReturnResult() ;
	return 
	    result ;
    } // EndMethod readGraphFromXML  

    public DirectedGraphWithDict readGraphFromXML  

	( DirectedGraphWithDict aNewReceiver ,
	  DirectedGraphWithDict aNewContext  ,
	  java.io.InputStream   anXMLSource  )

	throws 
	javax.xml.parsers.ParserConfigurationException
	,org.xml.sax.SAXException
	,java.io.IOException

    {
	receiver = aNewReceiver ;
	context  = aNewContext  ;
	return readGraphFromXML(anXMLSource) ;

    } // EndMethod readGraphFromXML

    private AbstractXMLReader getCurrentXMLReader() {
	return (AbstractXMLReader)internAdaptater.getContentHandler() ;
    }

    /**
     * Does nothing by default. This method
     * should initialize all variable needed to construct
     * the graph when the XML tags are reified in the <code>ContentHandler</code>
     * interface of <code>AbstractXMLReader</code>. This method is provided
     * as a "hook" to be able to construct several graphs within one
     * parsing, or to be able to switch the actual XML reader to which
     * the graph construction is delegated during the parsing.
     *
     * @see #switchToNewXMLReader  (AbstractXMLReader)
     * @see #readGraphFromXML      (java.io.InputStream)
     */
    protected void initGraphConstruction() {}

    /**
     * This method should returned the {@link DirectedGraphWithDict}
     * that was constructed from the XML output. This method is
     * provided as a "hook" to be able to construct several graphs
     * within one parsing, or to be able to switch the actual XML
     * reader to which the graph construction is delegated during the
     * parsing. Returns null by default.
     *
     * @see #switchToNewXMLReader(AbstractXMLReader)
     * @see #readGraphFromXML(java.io.InputStream)
     */
    protected DirectedGraphWithDict endGraphConstructionAndReturnResult() { return null ; }

    protected DirectedGraphWithDict receiver = null ;
    protected DirectedGraphWithDict context  = null ;

    /**
     * This method transput attributes contained in a 
     * {@link org.xml.sax.Attributes} into a {@link PropertyBag}
     */
    static protected void putSAXAttributesInPropertyBag
	( org.xml.sax.Attributes someAttributes,
	  PropertyBag            aProperyBag ) 
    {
	if (someAttributes==null) return ;
	for(int i=0; i<someAttributes.getLength() ; i++) {

	    aProperyBag.setPropertyToValue
		( someAttributes.getLocalName(i), 
		  someAttributes.getValue(i)
		  ) ; // EndCall setPropertyToValue

	} // EndFor Loop through someAttributes

    } // EndMethod putSAXAttributesInPropertyBag


} // EndClass AbstractXMLReader
