/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * CallLinkRenderer.java
 * 
 * Time-stamp: <2003-03-10 13:03:31 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: CallLinkRenderer.java,v 1.9 2008/10/30 14:53:14 ftaiani_normal Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import laas.friends.cosmopen.utils.* ;

/**
 * Renders the invocation links of a {@link CallGraph} in dot format.
 *
 * @see PropertyRenderer
 * @see DirectedGraphWithDict2DotOStream
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.9 $
 * @since Version 0.1.0
 */
public class CallLinkRenderer 
    implements PropertyRenderer {

    private String                kindOfLink ;
    private PropertyBag           propOfLink ;
    private NamedEdge             linkToBeRendered ;
    private DirectedGraphWithDict contextGraph ;

    private String       nameOfLink ;
    private String       threadIDs ;
    private StringBuffer result ;

    public String formatProperty (String       aKindOfLink,
				  PropertyBag  aPropOfLink,
				  NamedEdge    aLinkToBeRendered,
				  DirectedGraphWithDict aContextGraph )  
    {
	// COMMON VARIABLES USED BY ALL FORMATTING METHOD
	
	result = new StringBuffer() ;

	kindOfLink       = aKindOfLink;
	propOfLink       = aPropOfLink;
	linkToBeRendered = aLinkToBeRendered ;
	contextGraph     = aContextGraph;
	
	threadIDs  = propOfLink.provideProperty    ("thread") ;
	nameOfLink = propOfLink.getValueForProperty("method") ;

	if (nameOfLink==null)
	    nameOfLink =  propOfLink.provideProperty("name") ;

	// FORMATTING NAME ACCORDING TO NATURE OF INVOCATION

	if (kindOfLink.equals("create")) {
            NamedVertex createdClass = (NamedVertex)linkToBeRendered.getSink() ;
	    nameOfLink = "new\\n"+MiscUtils.GetShortEntityName(createdClass.getName()) ;
	} // Call to an object constructor

	if (kindOfLink.equals("threadCreate")) {
	    nameOfLink="creates t"
		+propOfLink.getValueForProperty("newThread")
		+":\\n"+nameOfLink ;
	} // Thread creation

	renderKindOnBuffer( kindOfLink, result ) ;

	// Removing edge label for functions

	String nameOfSink = ((NamedVertex)linkToBeRendered.getSink()).getName() ;
	if (nameOfSink.equals(nameOfLink)) nameOfLink="" ;

	// SPECIFIC PART OF THE FORMATTING

	if (CallGraphDotWriter.doUmlLike)  
	    umlFormatProperty() ;
	else
	    defaultFormatProperty() ;

	return result.toString() ;
	
    } // EndMethod formatProperty

    private void umlFormatProperty() 
    {
	String callOrder    = propOfLink.provideProperty ("callOrder");
	String globalOrder  = propOfLink.provideProperty ("globalOrder");

	// LABEL OF THE DOT LINK

	result.append(  "label=\"" ) ; 
	result.append(CallGraph.ConstructInvocationString(globalOrder,
							  threadIDs,
							  nameOfLink)) ;
	result.append(  "\" " ) ;
	
// 	// TAILLABEL

// 	if (callOrder.length()>0) 
// 	    result.append( "taillabel=\"" + callOrder + "\" " ) ;
	    
    } // EndMethod umlFormatProperty

    private void defaultFormatProperty() 
    {
	String incomingOrder = propOfLink.provideProperty ("incomingOrder");
	String callOrder     = propOfLink.provideProperty ("callOrder");
	String precedent     = propOfLink.provideProperty ("precedent");

	String taillabel ;

	if ((callOrder.length()>0) && (precedent.length()>0))
	     taillabel = precedent + ":" + callOrder ;
	else taillabel = precedent +       callOrder ;
	
	result.append(  "label=\"" + threadIDs ) ;
	if ((nameOfLink.length()>0) && (threadIDs.length()>0) )
	    result.append(  ":" ) ;
	if (nameOfLink.length()>0)
	    result.append(  nameOfLink ) ;

	String incomingOnLabel = "" ;
	String taillabelOnLabel= "" ;

	if (contextGraph.getDegree(linkToBeRendered.getSource())>5)
	    taillabelOnLabel = "["+taillabel ;
	if (contextGraph.getDegree(linkToBeRendered.getSink())>5)
	    incomingOnLabel = incomingOrder+"]" ;
	if ((taillabelOnLabel.length()>0) || (incomingOnLabel.length()>0))
	    result.append(  " "+taillabelOnLabel+"/"+incomingOnLabel ) ;
	if (contextGraph.getOutDegree(linkToBeRendered.getSource())>3) {
	    if (propOfLink.hasProperty("isStart"))
		result.append(  "->" ) ;
	    if (propOfLink.hasProperty("isReturn"))
		result.append(  "<-" ) ;
	}
	result.append(  "\" " ) ;

	if (propOfLink.hasProperty("isLeaf"))
	    result.append( "arrowhead=dot " ) ;

	if ( (propOfLink.hasProperty("isSameObject")) &&
	     (!linkToBeRendered.getSource().equals(linkToBeRendered.getSink()) )
	     )
	    result.append( "arrowtail=inv " ) ;

	if (taillabel.length()>0) {
	    result.append( "taillabel=\"" + taillabel + "\" " ) ;
	}

	if (incomingOrder.length()>0)
	    result.append(  "headlabel=\"" + incomingOrder + "\" ") ;

	result.append("labelangle=28, labeldistance=1.7 ") ;

    } // EndMethod formatProperty

    /**
     * When the link is an objet construction or thread creation, this
     * method reflects it on the dot formatting. Depending on the type
     * of graph (a raw {@link CallGraph} made of call trees or a
     * condensed graph), the information of the nature of the link is
     * not stored in the same property. In a raw call graph ({@link
     * CallGraph}), all links have the kind "controlflow" (property
     * 'kind'), and the marking of objet construction or thread
     * creation is done in the property 'subkind'. In a condensed
     * graph, objet construction and thread creation are stored in the
     * property 'kind'. Because this method is reused in {@link
     * ControlFlowRenderer}, which is used for raw graph, this had to
     * be taken out of {@link #formatProperty(String, PropertyBag,
     * NamedEdge, DirectedGraphWithDict) formatProperty}.
     */
    protected void renderKindOnBuffer(String kindOfLink, StringBuffer buffer) {

	// Special case of constructor

	if (kindOfLink.equals("create")) {
	    buffer.append("style=\"bold\" fontname=\"Times-Italic\" ") ;
	} // Call to an object constructor
				  
	if (kindOfLink.equals("threadCreate")) {
	    buffer.append("style=\"dotted\" fontname=\"Times-Italic\" ") ;
	} // Call to an object constructor

    } // EndMethod renderKind

} // EndClass CallLinkRenderer
