/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * DirectedGraphWithDict2DotOStream.java
 * 
 * Time-stamp: <2002-08-27 16:01:01 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: DirectedGraphWithDict2DotOStream.java,v 1.13 2007/09/18 18:23:42 ftaiani_normal Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import laas.friends.cosmopen.utils.* ;

/**
 * This class writes a {@link DirectedGraphWithDict} to
 * a {@link java.io.OutputStream } in a dot format.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.13 $
 * @since Version 0.0.2
 */
public class DirectedGraphWithDict2DotOStream extends DGWriterAdapter {

    /**
     * common stuff to constructors in one place
     */
    private void init() {
	PropertyRenderer classDiagRender = new ClassDiagramRenderer() ;
	PropertyRenderer callRender      = new CallLinkRenderer    () ;
	PropertyRenderer controlRender   = new ControlFlowRenderer () ;
	this.registerRendererForKindOfLink( classDiagRender ,"usage"        ) ; 
	this.registerRendererForKindOfLink( classDiagRender ,"superclass"   ) ;
	this.registerRendererForKindOfLink( callRender      ,"call"         ) ;
	this.registerRendererForKindOfLink( callRender      ,"create"       ) ;
	this.registerRendererForKindOfLink( callRender      ,"threadCreate" ) ;
	this.registerRendererForKindOfLink( callRender      ,"threadEnd"    ) ;
	this.registerRendererForKindOfLink( controlRender   ,"controlflow"  ) ;
    } // EndMethod init

    public DirectedGraphWithDict2DotOStream() {
	init() ;
    } // EndConstructor

    public DirectedGraphWithDict2DotOStream(java.io.OutputStream anOStream) 
    {
	super(anOStream) ;
	init() ;
    } // EndConstructor DirectedGraphWithDict2DotOStream

    /**
     * calls 
     * {@link DGWriterAdapter#writeGraph(DirectedGraphWithDict) writeGraph}
     */
    public void writeGraphInDotFormat(DirectedGraphWithDict aGraphToWrite) {
	aGraphToWrite.prepareForVisualisation();
	writeGraphVertexSeparate(aGraphToWrite);
	// writeGraph(aGraphToWrite);
	aGraphToWrite.endOfVisualisation();
    } // EndMethod writeGraphToStream

    /**
     * contains the size of one page in inches. The 2 number must be
     * separated by a comma. By default it is null (i.e. dot default).
     */
//     private String _pageSize   = "30,20" ;
    private String _pageSize   = null ;

    /**
     * contains the size of the whole graph in inches. The 2 number must be
     * separated by a comma. By default it is null (i.e. dot default).
     */
//     private String _entireSize = "25,15" ;
    private String _entireSize = null ;

    /**
     * allows the user to customize the page size and the whole size
     * of the ps document generate by dot. The 2 strings are used to 
     * generate the rendering of the dot graph.<br>
     * The 2 strings contain 2 values "<breath>,<heigh>" which are comma
     * separated. For instance:
     * <code>setPageSizeAndEntireSize("30,20","25,15") ;</code>
     * By default the page size is 30x20, and the entire size 25x15.
     * The value null is allowed: it means that no change is to be 
     * made to the old value.
     */
    public void setPageSizeAndEntireSize(String pageSize, String entireSize)
    {
	if (pageSize  !=null) this._pageSize   = pageSize ;
	if (entireSize!=null) this._entireSize = entireSize ;

    } ; // EndMethod setPageSizeEntireSize

    /**
     * contains the comment which it print in a separated node.
     * null means no comment.
     */
    private String _comment  = null ;

    /**
     * declares a comment which will be printed in a separated node
     * on the graph. The content of the comment may include newlines,
     * either \n, \l or \r for centered, left and right justifications.
     */
    public void setComment(String aComment) {
	_comment = aComment ;
    } // EndMethod setComment

    static private boolean leftRightOrientation = false ;
    static private boolean skipPagination = false ;
    static private boolean landscapeSwitch = false ;
    static private boolean formatForScreen = true ;

    static public void setLeftRight(boolean someLROrientation) {
	leftRightOrientation = someLROrientation ;
    } // EndMethod setLeftRightOrientation

    static public void setSkipPagination(boolean newSkipPagination) {
	skipPagination = newSkipPagination ;
    } // EndMethod setPaginationSwitch

    static public void setFormatForScreen(boolean newFormatForScreen) {
	formatForScreen = newFormatForScreen ;
    } // EndMethod setFormatForScreen
    
    static public void setLandscape(boolean newLandscapeSwitch) {
	landscapeSwitch = newLandscapeSwitch ;
    } // EndMethod setPaginationSwitch
    
    protected void writeHeader(DirectedGraphWithDict graphToBeRendered) {

	_theStreamIWriteTo.println("digraph CosmOpen {") ;
	_theStreamIWriteTo.print(    "  graph                          \n " +
				     "    [                            \n ");

	if (!formatForScreen) {
	    if (!skipPagination) {
		if (_pageSize!=null)
		    _theStreamIWriteTo.print("      page = \""+_pageSize+"\"   \n ");

		if (_entireSize!=null)
		    _theStreamIWriteTo.print("      size = \""+_entireSize+"\" \n ") ;
	    } // EndIf skipPagination

	    if (landscapeSwitch)
		_theStreamIWriteTo.print("      orientation = \"landscape\" \n ") ;
	    else
		_theStreamIWriteTo.print("      orientation = \"portrait\" \n ") ;

	    _theStreamIWriteTo.print(    "      nodesep = \".08\"          \n " +
					 "      ranksep = \"1\"            \n ") ;
	    if (leftRightOrientation)
		_theStreamIWriteTo.print("      rankdir  = \"LR\"\n") ;

	    _theStreamIWriteTo.print(    "      center  = \"1\"            \n " +
					 "    ];                           \n " +
					 "  node                           \n " +
					 "    [                            \n " +
					 "      shape = \"box\"            \n " +
					 "      fontsize = \"12\"          \n " +
					 "      height = \".25\"           \n " +
					 "    ];                           \n " +
					 "   edge                          \n " +
					 "    [                            \n " +
					 "      fontsize = \"12\"          \n " +
					 "      labelfontsize = \"10\"     \n " +
					 "//      decorate=\"true\"        \n " +
					 "    ] ;                          \n "
					 ) ; // EndCall print
	} // EndIf !formatForScreen
	else {
	    _theStreamIWriteTo.print("        nodesep = \"0.5\"" +
				     "        ranksep = \"0.3\"" +
				     "        center  = \"1\"" +
				     "        minlen=0" +
				     "     ];" +
				     "   node" +
				     "     [" +
				     "       shape = \"box\"" +
				     "       fontsize = \"16\"" +
				     "       height = \"0.2\"" +
				     "     ];" +
				     "    edge" +
				     "     [" +
				     "       fontsize = \"16\"" +
				     "       labelfontsize = \"10\"" +
				     "     ] ;"
				     ) ;
	} // EndIfElse !formatForScreen
	if (this._comment!=null)
	    _theStreamIWriteTo.print(
				     "   Comment [shape=\"record\",label=\""+
				     _comment +
				     "\" style=\"bold\"] ; "
				     ) ;
    } // EndMethod writeHeader

    protected void writeClosing() {
	_theStreamIWriteTo.println("}") ;
    } // EndMethod writeClosing

    protected static boolean useShortEntityName = false ;

    public static void setUseShortName(boolean newValue) { useShortEntityName = newValue ; }

    /**
     * renders the properties of the passed {@link NamedVertex } with
     * Dot class formating values.
     */
    protected void writeVertexRendering(NamedVertex classToBeRendered) {

	String name = 
	    theGraphBeingWrittenOut()
	    .getHumanFriendlyName(classToBeRendered,useShortEntityName) ;

	String renderingResult = 
	    "   \""+ classToBeRendered.getName() +"\" " +
	    "[ label=\"" + breakDownString(name) + "\" " ;

	if ( classToBeRendered.getPropBag().hasProperty("pseudoClass") )
	    renderingResult+="shape=\"ellipse\" fontsize = \"08\" ";

	if ( classToBeRendered.getPropBag().hasProperty("isRoot") )
	    renderingResult+="peripheries=2 " ;

	if ( classToBeRendered.getPropBag().hasProperty("isFunction") )
		renderingResult+="style=rounded " ;

	renderingResult+="] ;" ;

	_theStreamIWriteTo.println(renderingResult) ;

    } // EndMethod writeVertexRendering

    /**
     * renders the properties of the passed {@link NamedEdge } with
     * Dot edge formating values.
     */
    protected void writeEdgeRendering(NamedEdge linkToBeRendered) {

	_theStreamIWriteTo.print
	    (  "   \""+
	       ((NamedVertex)linkToBeRendered.getSource()).getName() + "\" -> \""+
	       ((NamedVertex)linkToBeRendered.getSink  ()).getName() + "\""
	       ) ; // EndCall println

	_theStreamIWriteTo.print  (  " [ " ) ;

	PropertyBag propOfLink = linkToBeRendered.getPropBag() ;
	String kind = propOfLink.getValueForProperty("kind") ;
	PropertyRenderer renderer = (PropertyRenderer)_edgeRendererMap.get(kind) ;

	if (renderer!=null)
	    _theStreamIWriteTo.print
		( renderer.formatProperty ( kind, 
					    propOfLink, 
					    linkToBeRendered , 
					    theGraphBeingWrittenOut() 
					    ) + " " ) ; 

	_theStreamIWriteTo.print(  "] " ) ;

	_theStreamIWriteTo.println(";") ;

    } // EndMethod writeEdgeRendering

    /**
     * Mapping between the registered {@link PropertyRenderer} and
     * the property names they're associated to.
     * @see #registerRendererForKindOfLink(PropertyRenderer,String)
     */
    private java.util.Map _edgeRendererMap = new java.util.HashMap() ;

    /**
     * Registers a property renderer, which will be called in 
     * {@link #writeEdgeRendering(NamedEdge)} if edge is of the passed kind
     * @see PropertyRenderer
     */
    public void registerRendererForKindOfLink(PropertyRenderer aPropRenderer,
					       String           aKind ) {
	_edgeRendererMap.put( aKind, aPropRenderer) ;
    }

    /**
     * breacks down a long string which contains commata (',')
     * and nested C++ template braces '<' and '>'. For example
     * the following string:
     * <PRE>
     *   "Test1&lt;test2&lt;test2&lt;test2,test3>,test3>,test3>"
     * <PRE>
     * will be broken down in the following one
     * </PRE>
     *   "Test1&lt;
     *       test2&lt;
     *         test2&lt;test2,test3>,test3>,
     *       test3>
     * </PRE>
     */
    static String 
	breakDownString(String input ) 
    {
	boolean wasBroken = false ;

	String indentUnit = "   " ;
	String indent     = "" ;

	if (input.length() < 25 ) return input ; // No breaking needed

	int countOnLine = 0 ;

	StringBuffer result = new StringBuffer("") ;

	for (int i=0; i<input.length() ; i++ ) {
	    result.append(input.charAt(i)) ;
	    countOnLine++ ;
	    if ((input.charAt(i)==',') && (countOnLine>10) ) {
		result.append("\\l").append(indent);
		wasBroken = true ;
		countOnLine = 0 ;
	    }
	    if (input.charAt(i)=='<') {
		indent = indentUnit + indent ;
		result
		    .append("\\l")
		    .append(indent) ;
		wasBroken = true ;
		countOnLine = 0 ;
	    } // EndIf

	    if (input.charAt(i)=='>') {
		indent = indent.substring(indentUnit.length()) ;
	    } // EndIf
	} // EndFor travelling through input
	
	if (wasBroken) result.append("\\l") ;

	return result.toString() ;

    } // EndMethod breakDownString

    /**
     * Only for test purposes. Should not be used.
     */
    public static void main(String[] args) {

	System.out.println(breakDownString("Test1")) ;
	System.out.println(breakDownString("Test,Test2")) ;
	System.out.println(breakDownString("Test1<test2>")) ;
	System.out.println(breakDownString("Test1<test2,test3>")) ;
	System.out.println(breakDownString("Test1<test2<test2<test2,test3>,test3>,test3>"));
	System.out.println(breakDownString("Test1<test2<AAAAAAAAAAAAAAAAAAAAAA,BBBBBBBBBBBBBBBBBBB>,test3<CCCCCCCCCCCCCCCCCCCCCC,DDDDDDDDDDDDDDDDDDDDDDDDDDD>>"));


    } // EndMain

} // EndClass DirectedGraphWithDict2DotOStream

