/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * DirectedGraphWithDictImpl.java
 * 
 * Time-stamp: <2003-03-07 22:18:02 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: DirectedGraphWithDictImpl.java,v 1.10 2007/09/18 18:23:42 ftaiani_normal Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import java.util.* ;
import salvo.jesus.graph.algorithm.DepthFirstDirectedGraphTraversal ;
import salvo.jesus.graph.Vertex ;

import laas.friends.cosmopen.utils.PropertyBag ;
import laas.friends.cosmopen.utils.MiscUtils ;

/**
 * This class adds dictionnariy facilities for
 * {@link NamedVertex } vertices to a
 * a {@link salvo.jesus.graph.DirectedGraphImpl}
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.10 $
 * @since Version 0.0.1
 */
public class DirectedGraphWithDictImpl
    extends salvo.jesus.graph.DirectedGraphImpl
    implements DirectedGraphWithDict
{

    /* [TODO] ftaiani 2001-06-29: add a method retrieveLinkForSourceSinkName may
       be a good idea */

    // 2001.05.30 ftaiani: Dictionary is obsolete. Switching to Map.
    // private java.util.Dictionary _classDataDict = new java.util.Hashtable() ;
    // [CHANGE] (Francois Taiani) Sat Jun 14 2003 21:02:00
    // TreeMap changed to HashMap. The order property on keys is not
    // of prime importance and seems to cause performance overhead.
    // protected SortedMap _classDataDict = new TreeMap() ;

    protected final Map         _classDataDict ;
    protected final PropertyBag _propBag       ;

    /**
     * Plain constructor.
     */
    public DirectedGraphWithDictImpl() {
	_classDataDict = new HashMap() ;
	_propBag       = new PropertyBag() ;
    } // EndConstructor 

    /**
     * Copy constructor.
     */
    public DirectedGraphWithDictImpl(DirectedGraphWithDictImpl toBeCopied) {
	super(toBeCopied) ;
	_classDataDict = new HashMap    (toBeCopied._classDataDict) ;
	_propBag       = new PropertyBag(toBeCopied._propBag) ;
    } // EndConstructor copy constructor

    /**
     * Clones the graph. The lists of vertices & edges are also cloned, so
     * that the original and the clone can be modified independently,
     * but the vertices and edges themselves are not cloned !
     */
    public Object clone() {
	return new DirectedGraphWithDictImpl(this) ;
// 	DirectedGraphWithDict clone 
// 	    = getEmptyGraphOfSameType() ;

// 	// [CHANGE] (Francois Taiani) Wed Apr 17 2002 16:49:22
// 	// The following implementation does not work: A part of a 
// 	// graph state is hidden in the lib implementation and can't
// 	// be cloned directly.

// // 	/* The following members are not cloned
// // 	   addedgelistener;
// // 	   addvertexlistener;
// // 	   connectedSet;
// // 	   removeedgelistener;
// // 	   removevertexlistener;
// // 	    traversal;
// // 	*/

// // 	clone.edges          = new Vector( this.edges    );
// // 	clone.vertices       = new Vector( this.vertices );
// // 	clone._classDataDict = new TreeMap(this._classDataDict);

// 	java.util.Collection nodesToAdd = 
// 	    this.getNodeSWithRegExp("*") ; // Should be efficient

// 	// [OPTIMIZATION] This can certainly be made far more
// 	// efficient. This is a workaround because I don't totally
// 	// know how the salvo.jesus lib is implemented.

// 	clone.addNamedVertexS ( nodesToAdd  ) ;
// 	try {
// 	    clone.doEdgeCompletionInContext ( this ) ;
// 	} catch(Exception e) {
// 	    throw new RuntimeException(e.toString());
// 	}

// 	return clone ;

    } // EndMethod clone

    public boolean doesContainVertex(Vertex candidate) {
	return _classDataDict.containsValue(candidate) ;
    } // EndMethod doesContainVertex
    public int getOutDegree(Vertex candidate) {
	return 
	    this.getOutgoingEdges(candidate).size() ;
    } // EndMethod getOutDegree
    public int getInDegree(Vertex candidate) {
	return 
	    this.getIncomingEdges(candidate).size() ;
    } // EndMethod getInDegree
    public int getDegree(Vertex candidate) {
	return 
	    this.getOutgoingEdges(candidate).size() + 
	    this.getIncomingEdges(candidate).size() ;
    } // EndMethod getDegree

    /**
     * Add a collection a collection of {@link NamedVertex NamedVertexS } 
     * to the graph.
     */
    public void addNamedVertexS(java.util.Collection classDataS) {
	java.util.Iterator iter = classDataS.iterator() ;
	while (iter.hasNext()) {
	    this.add((NamedVertex)iter.next()) ;
	} // EndWhile Loop through NamedVertexs to be added.
    } // EndMethod addVerticeS

    /**
     * Add a {@link NamedVertex} instances to the {@link DirectedGraphWithDictImpl}.
     * The {@link NamedVertex} is added to the graph and to the dictionnary
     * using its name. Names must be unique within a graph. 
     * <br><b>[CHANGE SINCE 0.0.3]</b> In case a different
     * {@link NamedVertex} is already present in the graph with the same name
     * , no exception is thrown but nothing happens. The graph remains
     * unchanged.
     */
    public void add(salvo.jesus.graph.Vertex passedNamedVertex) 
    {
// 	System.err.println("add(salvo.jesus.graph.Vertex passedNamedVertex)") ;
	NamedVertex newNamedVertex = (NamedVertex)passedNamedVertex ;
	NamedVertex classDataWithSameName 
	    = getNodeWithName(newNamedVertex.getName()) ;
// 	System.err.println(" -> classDataWithSameName : "+classDataWithSameName ) ;
	if (classDataWithSameName!=null) {
	    // There is already a NamedVertex with the same
	    // name in the graph. We don't add any new NamedVertex
	    // but we complete the property with the possible new
	    // ones of the class passed as parameter.
	    if (classDataWithSameName!=newNamedVertex)
		classDataWithSameName.getPropBag()
		    .addBag(newNamedVertex.getPropBag()) ;
	    return ; 
	} // EndIf classDataWithSameName!=null

	try {
	    super.add(newNamedVertex) ;
// 	    System.err.println(" -> super.add OK" ) ; 
	} catch (Exception e) {
	    System.err.println(e) ;
	}// EndCatch

	_classDataDict.put(newNamedVertex.getName(),newNamedVertex) ;
// 	System.err.println(" -> _classDataDict.put OK" ) ;

    } // EndMethod add

    /**
     * We overwrite this method to prevent that the removal
     * of an edge which sink or souce is not in the graph
     * raises an exception.
     */
    public void removeEdge(salvo.jesus.graph.Edge e)
                throws java.lang.Exception
    {
	
	if (this.doesContainVertex(e.getVertexA()) &&
	    this.doesContainVertex(e.getVertexB()) )
	    super.removeEdge(e) ;
    } // EndMethod removeEdge

    /**
     * The returned Iterator now iterated in ascending name
     * order on the {@link NamedVertex }
     */
    public java.util.Iterator getVerticesIterator() {
	return _classDataDict.values().iterator() ;
    } // EndMethod getVerticesIterator

    /**
     * removes the edges from the graph which are 
     * provided by an iterator passed as parameter. This iterator
     * should contain only {@link NamedEdge }
     */
    public void removeEdgesWithIterator(java.util.Iterator iter) 
	throws Exception 
    {
	while (iter.hasNext()) {
	    NamedEdge currLink = (NamedEdge)iter.next() ;
// 	    System.err.println("Removing "+currLink);
	    this.removeEdge(currLink) ;
	} // EndWhile
    } // EndMethdod removeEdgesWithIterartor

    /**
     * returns an iterator which goes through all the edges of the
     * graph.
     */
    public java.util.Iterator getEdgeIterator() {
	return getEdgeIteratorKind(null) ;
    } // EndMethod getEdgeIterator

    /**
     * returns an iterator which goes through all the edges of the
     * graph of a certain kind. If the <code>kind</code> parameter
     * is null, returns all edges.
     */
    public java.util.Iterator getEdgeIteratorKind(String kind) {
	return new GraphWDEdgeIteratorWithKind(this,kind) ;
    } // EndMethod getEdgeIteratorKind

    public salvo.jesus.graph.Edge createEdge( Vertex v1, Vertex v2 ) {
	return new NamedEdge( v1, v2 );
    }

    public void addEdge(salvo.jesus.graph.Edge passedNamedEdge)
	throws Exception 
    {
	addEdgeReturnActualEdge(passedNamedEdge) ;
    }

    /**
     * When calling {@link #addEdge(salvo.jesus.graph.Edge)} it may be that that edge that
     * is actually added to the graph is not the passed one, but
     * an equivalent one, in order to account for vertex equality
     * based on unique names.
     */
    public NamedEdge addEdgeReturnActualEdge(salvo.jesus.graph.Edge passedNamedEdge)
	throws Exception
    {
	return addReturnEdgeAndStrictOrNot(passedNamedEdge, true) ;
    }

    /**
     * As {@link #addEdgeReturnActualEdge(salvo.jesus.graph.Edge)},
     * with the last parameter telling if adding an already existing
     * edge with different properties is allowed.  If iAmStrict ==
     * true, than an exception will be thrown if it happens.
     * Otherwise, the property sets are simply merged.
     */
    public NamedEdge addReturnEdgeAndStrictOrNot(salvo.jesus.graph.Edge passedNamedEdge, 
						boolean iAmStrict)
	throws Exception

    {
	NamedEdge receivedLink = (NamedEdge)passedNamedEdge ;

	NamedVertex source = (NamedVertex)receivedLink.getSource() ;
	NamedVertex sameAsSource
	    = getNodeWithName(source.getName()) ;
        if (sameAsSource != null) source = sameAsSource ;

	// IF A LINK WITH SAME NAME AS NEW LINK AND
	// BETWEEN SAME SOURCE AND SAME SINK ALREADY IN GRAPH, DO NOTHING
	// This test is crucial to avoid duplicate edges
	// (several times the same NamedEdge instance in a graph)
	// This is not insured by the implementation of
	// salvo.jesus.graph.DirectedGraphImpl.

	if (sameAsSource!=null) { // Source of new Edge is in Graph

	    // Here we can't use Vector.contains which is not base on
	    // the equals method but on reference comparison.

// 	    java.util.Vector edgesFromSource
// 		= this.getOutgoingEdges(sameAsSource) ;
// 	    if (edgesFromSource.contains(newNamedEdge) )
// 		return ; // The link to be added is already in the graph

	    java.util.Iterator edgesFromSourceIter
		= this.getOutgoingEdges(sameAsSource).iterator() ;

	    // LOOP THROUGH EDGES OF SOURCE

	    while (edgesFromSourceIter.hasNext()) {
		boolean currentLinkIsEqualToReceivedOne ;
		NamedEdge currentLink = (NamedEdge)edgesFromSourceIter.next() ;
		if (iAmStrict)
		    currentLinkIsEqualToReceivedOne =
			currentLink.equals(receivedLink) ;
		else
		    currentLinkIsEqualToReceivedOne = 
			currentLink.equalsLazy(receivedLink) ;
		if ( currentLinkIsEqualToReceivedOne ) {
		    currentLink.addAttributesFromLink(receivedLink) ;
		    return currentLink ; // The link to be added is already in the graph
		}
	    } // EndWhile Loop through edges from source
	} // EndIf Source of new Edge is in Graph

	NamedVertex sink = (NamedVertex)receivedLink.getSink() ;
	NamedVertex sameAsSink
	    = getNodeWithName(sink.getName()) ;
        if (sameAsSink != null) sink = sameAsSink ;

	NamedEdge newNamedEdge 
	    = new NamedEdge(source,sink,receivedLink.getName()) ;
	newNamedEdge.getPropBag().addBag(receivedLink.getPropBag()) ;
// 	System.err.println("\nIn addEdge : NEW LINK:"+
// 			   "\n  between : "+source+
// 			   "\n          : "+sink+
// 			   "\n  kind    : "+newNamedEdge
// 			                      .getPropBag().getValueForProperty("kind")+
// 			   "\n  name    : "+newNamedEdge.getName()+
// 			   "\n" ) ;
	super.addEdge(newNamedEdge) ;
	return newNamedEdge ;
    } // EndMethod addEdge

    public void remove(Vertex v)
	throws java.lang.Exception 
    {
	NamedVertex classDataToBeRemoved = (NamedVertex)v ;

	if (!doesContainVertex(classDataToBeRemoved))
	    return ; // No such class in Graph
	Iterator neighbors = this.getAdjacentVertices(v).iterator() ;
	super.remove(v) ;
	while(neighbors.hasNext()) {
	    NamedVertex currentNeighbor = (NamedVertex)neighbors.next() ;
//	    System.err.println("currentNeighbor:"+currentNeighbor);
	    if (currentNeighbor.isPseudo() &&
		this.vertexHasNoParentsOrNoChilds(currentNeighbor))

		this.remove(currentNeighbor) ;

	} // EndWhile Loop through Neighbors
	_classDataDict.remove(classDataToBeRemoved.getName()) ;
    } // EndMethod remove

    public boolean vertexHasNoParentsOrNoChilds(NamedVertex candidate) {

	if (!doesContainVertex(candidate))
	    return false ; // No such class in Graph
	
	return 
	    (this.getOutgoingEdges(candidate).size()==0) ||
	    (this.getIncomingEdges(candidate).size()==0) ;
    } // EndMethod hasNoParentsOrNoChilds

    public void cleanAroundVertex(NamedVertex needsRoom) 
	throws java.lang.Exception 
    {
	if (!doesContainVertex(needsRoom))
	    return ; // No such class in Graph
	Iterator neighbors = this.getAdjacentVertices(needsRoom).iterator() ;
	while(neighbors.hasNext()) {
	    NamedVertex currentNeighbor = (NamedVertex)neighbors.next() ;

	    if (this.getDegree(currentNeighbor)==1)

		this.remove(currentNeighbor) ;

	} // EndWhile Loop through Neighbors
    } // EndMethod cleanAroundVertex

    public void addGraph(DirectedGraphWithDict aGraphToAdd) 
	throws java.lang.Exception 
    {
	if (aGraphToAdd==null) return ;
	java.util.Iterator iterOnNodes = aGraphToAdd.getVerticesIterator() ;

	// LOOP THROUGH VERTICES OF THE GRAPH TO BE ADDED

	while (iterOnNodes.hasNext()) {
	    NamedVertex currentNamedVertex = (NamedVertex)iterOnNodes.next() ;
	    this.add(currentNamedVertex) ;
	    java.util.Iterator iterOnEdges 
		= aGraphToAdd.getOutgoingEdges(currentNamedVertex).iterator() ;

	    // LOOP THROUGH EDGES OF THE CURRENT VERTEX

	    while (iterOnEdges.hasNext()) {
		// Here there is a trick: We don't need to explicitely
		// add the sink of the edge to the 'this' graph, since
		// addEdge automatically do it if needed.
		this.addEdge((NamedEdge)iterOnEdges.next()) ;
	    } // EndWhile Loop through edges of current vertex
	    
	} // EndWhile Loop through vertices
    } // EndMethod addGraph

    /**
     * Searches for a {@link NamedVertex } with the passed name
     * in the {@link DirectedGraphWithDictImpl }. If none is found
     * null is returned.
     * @see #provideNamedVertexWithName(String aName)
     */
    public NamedVertex getNodeWithName(String aName) {
	return (NamedVertex)_classDataDict.get(aName) ;
    } // EndMethod getNodeWithName

    /**
     * {@link #provideNamedVertexWithName(String aName)} may create {@link NamedVertex} in
     * the graph. By overwriting this method it's possible to change
     * the type these implicitly created vertices to a subclass
     * of {@link NamedVertex}). By default it returns a new instance
     * of {@link NamedVertex}.
     */
    public NamedVertex createNewNameVertexWithName(String aName) {
	return  new NamedVertex(aName) ;
    } // EndMethod createNewNameVertexWithName

    /**
     * Searches is there is already a {@link NamedVertex } with the passed name
     * in the {@link DirectedGraphWithDictImpl }. If yes this {@link NamedVertex }
     * is returned. If no, a new {@link NamedVertex } is created
     * and added to the {@link DirectedGraphWithDictImpl}
     * with the corresponding name. This new vertex is created using
     * {@link #createNewNameVertexWithName(String)}, that can be
     * overwriten.
     *
     * @see #getNodeWithName(String aName)
     * @see #createNewNameVertexWithName(String)
     */
    public NamedVertex provideNamedVertexWithName(String aName) {
	NamedVertex result ;
// 	System.err.println("provideNamedVertexWithName") ;
	result = getNodeWithName(aName) ;
// 	System.err.println(" -> result: "+result );
	if (result==null) {
	    result = createNewNameVertexWithName(aName) ;
// 	    System.err.println(" -> result: "+result );
	    add(result) ;
// 	    System.err.println(" -> provideNamedVertexWithName { add } OK" );
	} // EndIf no NamedVertex with such a name yet
	return result ;
    } // EndMethod provideNamedVertexWithName

    /**
     * goes through all {@link NamedVertex } of the graph and retains
     * the ones whose name matches the passes wilcard expression. For
     * instance <code>getNodeSWithRegExp("*")</code> returns all
     * {@link NamedVertex } in the graph, and
     * <code>getNodeSWithRegExp("*foo")</code> returns all {@link
     * NamedVertex } whose name ends with foo. Complex queries like
     * 'x*y*z' are not allowed and raise an exception.<p>

     * <b>(O)ptimization</b>: This method is often used to apply a
     * given action to a set of nodes (removing, adding, selecting,
     * ... etc). Returning a <code>SortedSet</code> and iterating on
     * it is straightforward but inefficient. If needed performance
     * can be improved by using a call back method (or rather object)
     * to have the same action applied on all matching nodes without
     * going through an intermediate sorted set.
     */
    public java.util.SortedSet getNodeSWithRegExp(String limitedWildcardExp ) {

	// We make a copy of the passed string for possible error messages.
	String stringUnderWork = new String(limitedWildcardExp) ;

	if (stringUnderWork.equals(""))
	    // Returning empty set
	    return new java.util.TreeSet(); 
	if (stringUnderWork.equals("*")) 
	    // Retuning all vertices of the graph
// 	    return new java.util.TreeSet(this.cloneVertices()) ;
 	    return new java.util.TreeSet(this._classDataDict.values()) ;

	boolean beginsWithStar 
	    = (stringUnderWork.charAt(0)=='*') ;
	if (beginsWithStar) // removes starting star
	    stringUnderWork = stringUnderWork.substring(1) ;

	boolean   endsWithStar 
	    = (stringUnderWork.charAt(stringUnderWork.length()-1)=='*') ;
	if (endsWithStar)  // removes ending star
	    stringUnderWork 
		= stringUnderWork.substring(0,stringUnderWork.length()-1) ;

	if ( (stringUnderWork.equals("")) ||
	     (stringUnderWork.indexOf("*")!=-1) )
	     throw new DirectedGraphWithDict.NotALimitedWilcardExpression
		 ("'" + limitedWildcardExp + "' is ill formed.") ;

	// We use four booleans to remember which test to apply
	// to each NamedVertex in the graph. Note that this code
	// is certainly not the most elegant.

	boolean testStarts   = !beginsWithStar &&  endsWithStar ;
	boolean testEnds     =  beginsWithStar && !endsWithStar ;
	boolean testContains =  beginsWithStar &&  endsWithStar ;
	boolean testEquals   = !beginsWithStar && !endsWithStar ;

	java.util.Iterator iterator = this.getVerticesIterator() ;
	
	java.util.SortedSet result = new java.util.TreeSet();

	while (iterator.hasNext() ) {
	    NamedVertex currentClassD  = (NamedVertex)iterator.next() ;
	    String    currentName    = currentClassD.getName() ;
	    boolean   isPartOfResult = false ;

	    if (testStarts) 
		isPartOfResult = currentName.startsWith(stringUnderWork) ;
	    if (testEnds) 
		isPartOfResult = currentName.endsWith(stringUnderWork) ;
	    if (testContains) 
		isPartOfResult = (currentName.indexOf(stringUnderWork)!=-1) ;
	    if (testEquals) 
		isPartOfResult = currentName.equals(stringUnderWork) ;

	    if (isPartOfResult) result.add(currentClassD) ;

	} // EndWhile loop through all NamedVertex of the graph

	return result ;

    } // EndMethod getNodeSWithRegExp

    public void 
	doEdgeCompletionInContext(DirectedGraphWithDict context) 
    throws java.lang.Exception {
	java.util.Iterator iter = this.getVerticesIterator() ;

	// Loop through nodes of the 'this' graph
	while(iter.hasNext()) { 

	    NamedVertex currentNode = (NamedVertex)iter.next() ;
	    NamedVertex sameNameInContext 
		= context.getNodeWithName(currentNode.getName()) ;

	    if (currentNode!=sameNameInContext) 
		// There is no NamedVertex in the context graph with the name
		// of the current node, or there is a NamedVertex with that name 
		// but this is not the same. In any case we don't proceed further
		// and we switch to the next NamedVertex item.
		continue ;

	    java.util.Collection outGoingLinksInContext 
		= context.getOutgoingEdges(currentNode) ;

	    java.util.Iterator linkIter = outGoingLinksInContext.iterator() ;

	    // Loop through links of the currentNode in the context
	    while(linkIter.hasNext()) {
		NamedEdge currentLink          = (NamedEdge)linkIter.next() ;
		NamedVertex sinkOfLinkInContext = (NamedVertex)currentLink.getSink() ;
		if ( this.getNodeWithName(sinkOfLinkInContext.getName())
		     == sinkOfLinkInContext) {
		    this.addEdge(currentLink) ;
		} // EndIf sinkOfLink 
		
	    } // EndWhile Loop through links of the currentNode in the context
	    
	} // EndWhile Loop through nodes of the 'this' graph
    } // EndMethod doEdgeCompletionInContext

    /** *******************************************/
    public boolean equals(Object obj) {
	return equalsWithPossibleComments(obj, false) ;
    } // EndMethod equals

    /** *******************************************
     * As {@link #equals(Object)} except that in case
     * the graphs are no equal, some diagnostic is printed.
     */
    public boolean equalsDebug(Object obj) {
	return equalsWithPossibleComments(obj, true) ;
    } // EndMethod equals

    /** *******************************************/
    private boolean equalsWithPossibleComments(Object obj, boolean commentsAreWished ) {

	if (this==obj) return true ; // trivial case
	if (!(obj instanceof DirectedGraphWithDictImpl)) return false ;

	DirectedGraphWithDict autreGraph = (DirectedGraphWithDict)obj ;

	if (autreGraph.getVerticesCount()!=this.getVerticesCount()) {
	    if (commentsAreWished)
		System.err.println("<> because <> counts. this "
				   + this      .getVerticesCount() + " autre "
				   + autreGraph.getVerticesCount() ) ;
	    return false ;
	} // EndIf

	// both graphs have the same number of vertices

	if (!getPropBag().equals(autreGraph.getPropBag())) {
	    if (commentsAreWished)
		System.err.println("<> because <> top level properties. this "
				   + this      .getPropBag() + " autre "
				   + autreGraph.getPropBag() ) ;
	    return false ;
	}

	java.util.Collection verticesThis  = this      .cloneVertices() ;
	java.util.Collection verticesAutre = autreGraph.cloneVertices() ;
	java.util.Iterator iterOnThis  = verticesThis.iterator() ;

	// LOOP THROUGH ALL NODES OF THE CURRENT (this) GRAPH

	while (iterOnThis.hasNext()) {
	    NamedVertex currentNode = (NamedVertex)iterOnThis.next() ;

	    if ( autreGraph
		   .getNodeWithName
		     ( currentNode.getName() ) == null
		 ) {
		if (commentsAreWished)
		    System.err.println("<> because ' " + currentNode + " in this not in autre.") ;
		return false ;
	    }
	    NamedVertex autreCurrentNode = autreGraph
		   .getNodeWithName
		( currentNode.getName() ) ;

	    // NOTE: We assume there are no duplicate edges. That means
	    // that the same edge instance is not registered twice in
	    // the graph. This must be insured by the remaining implementation
	    // of the class, in particular addEdge(Edge).

	    java.util.SortedSet edgesInThis 
		= new java.util.TreeSet
		    (      this.getIncomingEdges(currentNode)) ;

	    java.util.SortedSet edgesInAutre
		= new java.util.TreeSet
		    (autreGraph.getIncomingEdges(currentNode)) ;

	    if (edgesInThis.size()!=edgesInAutre.size()) {
		if (commentsAreWished)
		    System.err.println( "<> because " + currentNode + " has <> degrees: \n"+
					" degree in this : " + edgesInThis .size() + " : " 
					+ edgesInThis  + "\n" +
					" degree in autr : " + edgesInAutre.size() + " : " 
					+ edgesInAutre + "\n") ;		
		return false ;
	    }

	    java.util.Iterator iterOnEdgeOfThis =
		edgesInThis .iterator() ;
	    java.util.Iterator iterOnEdgeOfAutre =
		edgesInAutre.iterator() ;

	    while(iterOnEdgeOfThis.hasNext()) {
		NamedEdge linkThis  = (NamedEdge)iterOnEdgeOfThis .next() ;
		NamedEdge linkAutre = (NamedEdge)iterOnEdgeOfAutre.next() ;
		if (!linkThis.equals(linkAutre)) {
		    if (commentsAreWished)
			System.err.println
			    ( "<> because ' 2 links <> " + linkThis + " <> " + linkAutre +
			      " linkThis  " + linkThis .getPropBag() +
			      " linkAutre " + linkAutre.getPropBag() ) ;
		    return false ;
		}
	    } // EndWhile Lopps through incoming edges of this and autre

	} // EndWhile Loop through the vertices of the this graph
	
	return true ;

    } // EndMethod equals


    /**
     * This methods extracts the subgraph obtained when traversing
     * the graph along the directed edges in the reverse direction
     * from the sink vertex to the source from a given starting vertex.
     */
    public DirectedGraphWithDict
	extractBackwardClosure(NamedVertex aStartingPoint)
    {
	return extractBackwardClosureFromArray(new NamedVertex[]{aStartingPoint}) ;
    } // EndMethod extractBackwardClosure


    /**
     * This methods extracts the subgraph obtained when traversing
     * the graph along the directed edges (in the normal direction
     * source to sink) from a given starting vertex.
     */
    public DirectedGraphWithDict
	extractForwardClosure(NamedVertex aStartingPoint) 
    {
	return extractForwardClosureFromArray(new NamedVertex[]{aStartingPoint}) ;
    } // EndMethod extractForwardClosure


    public DirectedGraphWithDict 
	extractBackwardClosureFromGraph(DirectedGraphWithDict aStartingGraph)
    {
	// [CHANGE] ftaiani Wed Jul  4 2001 14:59:19
// 	NamedVertex[] dummy = new NamedVertex[0] ;
// 	NamedVertex[] someStartingPointS 
// 	    = (NamedVertex[])aStartingGraph.cloneVertices().toArray(dummy) ;

// 	DirectedGraphWithDict result =
// 	    extractBackwardClosureFromArray(someStartingPointS) ;
// 	try {
// 	    result.addGraph(aStartingGraph) ;
// 	} catch (Exception e) {
// 	    e.printStackTrace() ;
// 	} // EndCatch
// 	return result ;

	return extractFixBackwardFromGraph( aStartingGraph, -1 ) ;
    } // EndMethod extractBackwardClosureFromGraph


    public DirectedGraphWithDict 
	extractForwardClosureFromGraph(DirectedGraphWithDict aStartingGraph)
    {
	NamedVertex[] dummy = new NamedVertex[0] ;
	NamedVertex[] someStartingPointS 
	    = (NamedVertex[])aStartingGraph.cloneVertices().toArray(dummy) ;
	DirectedGraphWithDict result =
	    extractForwardClosureFromArray(someStartingPointS) ;
	try {
	    result.addGraph(aStartingGraph) ;
	} catch (Exception e) {
	    e.printStackTrace() ;
	} // EndCatch
	return result ;
    } // EndMethod extractForwardClosureFromGraph


    public DirectedGraphWithDict
	extractForwardClosureFromArray(NamedVertex[] someStartingPointS) {
	
	return extractFixForwardFromArray(someStartingPointS,-1) ;

    } // EndMethod extractForwardClosureFromArray

    /************************************************
     * @param depthLimit -1 means no limit on the traversal depth.
     */
    public DirectedGraphWithDict
	extractFixForwardFromArray(NamedVertex[] someStartingPointS,
				    int         depthLimit ) 
    {
	
	DirectedGraphWithDict extractedGraph = getEmptyGraphOfSameType() ;
	salvo.jesus.graph.Visitor visitor 
	    = new ExtractingVisitor_( this, 
				      extractedGraph, 
				      someStartingPointS ) 
		{
		    protected Collection nextEdges(Vertex vertex) {
			return _sourceGraph.getOutgoingEdges(vertex) ;
		    } // EndCall nextEdges
		}; // EndClass ExtractingVisitor_

	BFMultiStartDirectedGraphTraversal
	    traversalEngine = 
	       new BFMultiStartDirectedGraphTraversal(this,true,depthLimit) ;
	traversalEngine.traverse(someStartingPointS, visitor) ;
	return extractedGraph ;

    } // EndMethod extractFixForwardFromArray

    public DirectedGraphWithDict
	extractBackwardClosureFromArray(NamedVertex[] someStartingPointS) {

	return extractFixBackwardFromArray(someStartingPointS,-1) ;

    } // EndMethod extractBackwardClosureFromArray

    /************************************************
     * @param depthLimit -1 means no limit on the traversal depth.
     */
    public DirectedGraphWithDict
	extractFixBackwardFromArray(NamedVertex[] someStartingPointS,
				    int         depthLimit ) 
    {
	
	DirectedGraphWithDict extractedGraph = getEmptyGraphOfSameType() ;
	salvo.jesus.graph.Visitor visitor 
	    = new ExtractingVisitor_( this, 
				      extractedGraph, 
				      someStartingPointS ) 
		{
		    protected Collection nextEdges(Vertex vertex) {
			return _sourceGraph.getIncomingEdges(vertex) ;
		    } // EndCall nextEdges
		}; // EndClass ExtractingVisitor_

	BFMultiStartDirectedGraphTraversal
	    traversalEngine = 
	       new BFMultiStartDirectedGraphTraversal(this,false,depthLimit) ;
	traversalEngine.traverse(someStartingPointS, visitor) ;
	return extractedGraph ;

    } // EndMethod extractFixBackwardFromArray

    // **********************************************

    public DirectedGraphWithDict 
	extractFixBackwardFromGraph(DirectedGraphWithDict aStartingGraph,
				    int                   depthLimit )
    {
	NamedVertex[] dummy = new NamedVertex[0] ;
	NamedVertex[] someStartingPointS 
	    = (NamedVertex[])aStartingGraph.cloneVertices().toArray(dummy) ;

	DirectedGraphWithDict result =
	    extractFixBackwardFromArray(someStartingPointS,depthLimit) ;
	try {
	    result.addGraph(aStartingGraph) ;
	} catch (Exception e) {
	    e.printStackTrace() ;
	} // EndCatch
	return result ;

    } // EndMethod  extractFixBackwardFromGraph


    // **********************************************
    public DirectedGraphWithDict 
	extractFixForwardFromGraph(DirectedGraphWithDict aStartingGraph,
				    int                  depthLimit ) 
    {
	NamedVertex[] dummy = new NamedVertex[0] ;
	NamedVertex[] someStartingPointS 
	    = (NamedVertex[])aStartingGraph.cloneVertices().toArray(dummy) ;

	DirectedGraphWithDict result =
	    extractFixForwardFromArray(someStartingPointS,depthLimit) ;
	try {
	    result.addGraph(aStartingGraph) ;
	} catch (Exception e) {
	    e.printStackTrace() ;
	} // EndCatch
	return result ;

    } // EndMethod extractFixForwardFromGraph


    /**
     * This class is a visitor for the travsersal of directed graph
     * which add the outgoing adjacent vertices and the corresponding edges
     * for each visited vertex to a resulting extrated graph under work
     * passed at construction. <P>
     * This class is abstract. The method {@link #nextEdges(Vertex vertex) }
     * needs to be implemented in order to use it.<P>
     * <B>Example</B><BR><PRE>
     * salvo.jesus.graph.Visitor visitor 
     *	= new ExtractingVisitor_( this, extractedGraph,aStartingPoint ) {
     *	    protected Vector nextEdges(Vertex vertex) {
     *		return _sourceGraph.getOutgoingEdges(vertex) ;
     *	    }
     *	};
     * </PRE>
     * @author Francois Taiani   <taiani@laas.fr>
     * @version 0.0.2 [under work]
     * @since Version 0.0.2
     * @see #nextEdges(Vertex vertex)
     */
     private abstract class ExtractingVisitor_ 
	implements salvo.jesus.graph.Visitor {

	private   DirectedGraphWithDict _resultUnderwork ;
	protected DirectedGraphWithDict _sourceGraph ;
	
	public ExtractingVisitor_
	    ( DirectedGraphWithDict sourceGraph,
	      DirectedGraphWithDict targetGraph,
	      NamedVertex[]           startingVertices )
	{
// 	    System.err.println () ;
// 	    System.err.println ("New ExtractingVisitor_") ;

	    _resultUnderwork = targetGraph ;
	    _sourceGraph     = sourceGraph ;

	    for(int i=0; i < startingVertices.length ; i++) {
		_resultUnderwork.add ( startingVertices[i] ) ;
	    } // EndFor i

// 	    System.err.print   ("   Adding Vertex :" ) ;
// 	    System.err.println (startingVertex) ;

	} // EndConstructor _ExtractingVisitor

	public boolean visit(salvo.jesus.graph.Vertex vertexToVisit) {
	    
//  	    System.err.print   ("Visited Vertex :" ) ;
//  	    System.err.println (vertexToVisit) ;

	    Collection outgoingEdges = nextEdges(vertexToVisit) ;

	    for(Iterator i = outgoingEdges.iterator() ; i.hasNext() ; ) {
		NamedEdge currentEdge = (NamedEdge)i.next() ;
// 		System.err.print   ("Visited Edge :" ) ;
// 		System.err.println (currentEdge) ;
// 		System.err.println ("Opposite Vertex:"+ 
// 				    currentEdge.getOppositeVertex(vertexToVisit)) ;

		_resultUnderwork.add
		    ( (NamedVertex)
		      currentEdge.getOppositeVertex(vertexToVisit) ) ;

// 		System.err.print   ("   Adding Vertex :" ) ;
// 		System.err.println (currentEdge.getOppositeVertex(vertexToVisit)) ;

		try {
		    _resultUnderwork.addEdge(currentEdge) ;
		} catch (Exception e) {
		    e.printStackTrace() ;
		} // EndCatch
	    } // EndFor

	    return true ;
	} // EndMethod visit(Vertex vertexToVisit)

	 /**
	  * This method determines how the visitors adds edges and vertices
	  * to the graph under construction. It must be consistent with the
	  * way the graph is actually traversed. During a traversal in which
	  * an {@link DirectedGraphWithDictImpl.ExtractingVisitor_ } is used, 
	  * the {@link salvo.jesus.graph.Edges } returned by this method
	  * are supposed to end or start on the passed <PRE>vertex</PRE> argument.
	  * These Edges are added to the extracted graph when the
	  * <PRE>vertex</PRE> argument is visited, along with the vertices
	  * which they link to the passed <PRE>vertex</PRE> argument.<P>
	  */
	protected abstract Collection nextEdges(Vertex vertex) ;

    } // EndClass ExtractingVisitor_

    public NamedEdge getLinkWithSourceSinkName(NamedVertex   source,
					      NamedVertex   sink  ,
					      String      name) {

	java.util.Iterator edgesFromSourceIter
	    = this.getOutgoingEdges(source).iterator() ;

	// LOOP THROUGH EDGES OF source

	while (edgesFromSourceIter.hasNext()) {
	    NamedEdge currentLink = (NamedEdge)edgesFromSourceIter.next() ;
	    if ( currentLink.getName().equals(name) && (currentLink.getSink()==sink) ) {
		return currentLink ; 
	    } // We've found a link with the searched sink and name
			
	} // EndWhile Loop through edges from source

	return null ; // Nothing found

    } // EndMethod getLinkWithSourceSinkName

    public DirectedGraphWithDict getEmptyGraphOfSameType() {
	try {
	    return (DirectedGraphWithDict)getClass().newInstance() ;
	} catch (Exception e) { 
	    e.printStackTrace(); System.exit(1) ; // That should not happen !
	}
	return null ; // Never reached.
    } // EndMethod DirectedGraphWithDict

    /**
     * The returned {@link DirectedGraphWriter} is of the same
     * type as the one returned by {@link #getRawWriter()}
     * in this case. This can be changed in subclasses.
     */
    public DirectedGraphWithDict2DotOStream getFormattingWriter() {
	return new DirectedGraphWithDict2DotOStream() ;
    }

    public DirectedGraphWithDict2DotOStream getRawWriter() {
	return new DirectedGraphWithDict2DotOStream() ;
    }

    public PropertyBag getPropBag() { return _propBag ; } ;

    public String getHumanFriendlyName(NamedVertex toBeNamed, 
				       boolean     removeNamespace) {
	if (removeNamespace)
	     return MiscUtils.GetShortEntityName(toBeNamed.getName()) ;
	else return toBeNamed.getName() ;

    } // EndMethod getHumanFriendlyName

    public int removeVertexWithIterator(java.util.Iterator iter) 
        throws Exception 
    {
        int nbRemovedVertices = 0 ;
        while (iter.hasNext()) {
            NamedVertex vertexToRemove = (NamedVertex)iter.next() ;
            this.remove(vertexToRemove) ;
            nbRemovedVertices++ ;
        } // EndWhile Loop through result
        return nbRemovedVertices ;
    } // EndMethod removeVertexWithIterator

    public void prepareForVisualisation() {}

    public void endOfVisualisation() {}

    public boolean isEdgeOrderProeminant() { return false ; }

} // EndClass DirectedGraphWithDictImpl

