/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * GraphWDEdgeIteratorWithKind.java
 * 
 * Time-stamp: <01/07/03 16:22:53 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: GraphWDEdgeIteratorWithKind.java,v 1.2 2003/04/07 18:10:36 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import java.util.Vector ;
import java.util.Iterator ;

/**
 * This class implements an iterator on the edges of a
 * {@link DirectedGraphWithDict }. The iterator is
 * able too take only edges of a certain kind.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.2 $
 * @since Version 0.0.4
 */
public class GraphWDEdgeIteratorWithKind implements Iterator {

    DirectedGraphWithDict _iteratedGraph ;
    String                _kindCriterion ;
    NamedVertex             _currentNamedVertex    = null ;
    NamedEdge              _currentEdge         = null ;
    Iterator              _iterOnNamedVertex     = null ;
    Iterator              _iterOnOutGoingEdges = null ;

    /**
     * creates an iterator on the edges of a {@link DirectedGraphWithDict }.
     * @param aGraph the graph the iterator work on.
     * @param aKind  a string representing the kind of links to be
     *               iterated on. If null, all edges are iterated.
     */
    public GraphWDEdgeIteratorWithKind(DirectedGraphWithDict aGraph,
				       String                aKind ) {
	_iteratedGraph   = aGraph ;
	_kindCriterion   = aKind ;
	_iterOnNamedVertex = aGraph.getVerticesIterator() ;

    } // EndConstructor GraphWDEdgeIteratorWithKind

    public boolean hasNext() {
	return (getCurrentEdge()!=null) ;
    } // EndMethod hasNext

    /**
     * without any consiteration of the kind criterion
     * says if there is still some edge ("of any kind")
     * available.
     */
    private boolean internalHasNext() {

	while (   containsSomething(_iterOnNamedVertex) &&
	        ! containsSomething(_iterOnOutGoingEdges)
	       ) 
	    {
// 		System.out.println( "internalHasNext: In while Loop.") ;
		_currentNamedVertex = (NamedVertex)_iterOnNamedVertex.next() ;
		_iterOnOutGoingEdges =
		    new Vector(
			       _iteratedGraph
			       .getOutgoingEdges(_currentNamedVertex)
			       ).iterator() ;
		
	} // EndWhile more NamedVertex and not yet Edges

// 	System.out.println( "internalHasNext return "+
// 			    containsSomething(_iterOnOutGoingEdges)) ;

	return containsSomething(_iterOnOutGoingEdges) ;

    } // EndMethod internalHasNext

    /**
     * returns true if the passed Iterator is not null
     * and contains something.
     */
    private boolean containsSomething(Iterator iter)  {
	if (iter!=null)
	    return iter.hasNext() ;
	return false ;
    } // EndMethod containsSomething

    private NamedEdge getCurrentEdge() {

	// ** LOOP THROUGH EDGES UNTIL EDGE OF GOOD KINF FOUND

	while(   (_currentEdge==null) && 
		 this.internalHasNext() ) 
	    {
		NamedEdge candidate = (NamedEdge)_iterOnOutGoingEdges.next() ;
		boolean isOfGoodKind = true ;

		if (_kindCriterion !=null) {
		    String kindOfCandidate =
			candidate.getPropBag().getValueForProperty("kind");
		    isOfGoodKind = _kindCriterion.equals(kindOfCandidate) ;
		} // EndIf kind criterion exist

		if (isOfGoodKind) _currentEdge = candidate ;

	    } /* EndWhile still No current edge found
		 but still some edges to search */

	return _currentEdge ;

    } // EndMethod getCurrentEdge

    public Object next() {
	NamedEdge result = getCurrentEdge() ;
	_currentEdge = null ;
	return result ;
    } // EndMethod next

    public void remove() 
	throws UnsupportedOperationException,
	       IllegalStateException
    {
	throw new UnsupportedOperationException() ;
    } // EndMethod remove

} // EndClass GraphWDEdgeIteratorWithKind
