/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * NamedEdge.java
 * 
 * Time-stamp: <02/08/13 23:05:34 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: NamedEdge.java,v 1.6 2004/10/01 15:12:54 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis  ;

import java.util.HashSet ;
import java.util.HashSet ;
import salvo.jesus.graph.Vertex ;
import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * This class represents a link tag extracted from an xml file
 * generated with hydrogen.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.6 $
 * @since Version 0.0.1
 */
public class NamedEdge 
extends salvo.jesus.graph.DirectedEdgeImpl implements Comparable {

    private PropertyBag _propertyContainer ;
    private String      _name = "" ;
    private final int   _hashCode ;

    public NamedEdge ( NamedVertex sourceVertex, NamedVertex sinkVertex,
		       String      name ) 
    {
	super( sourceVertex,  sinkVertex ) ;
        _hashCode = sourceVertex.hashCode() ^ sourceVertex.hashCode() ; // XOR
	_propertyContainer = new PropertyBag() ;
	this.setPropertyToValue ("target"   ,sinkVertex.getName() ) ;
	setName(name);
    } // EndConstructor NamedEdge

    public NamedEdge (Vertex sourceVertex, Vertex sinkVertex) {
	super( sourceVertex,  sinkVertex ) ;
        _hashCode = sourceVertex.hashCode() ^ sourceVertex.hashCode() ; // XOR
	_propertyContainer = new PropertyBag() ;
	setPropertyToValue ("target", ((NamedVertex)sinkVertex).getName() ) ;
    } // EndConstructor NamedEdge

    /**
     * returns the property bag which contains
     * the properties attached to that NamedVertex
     */
    public PropertyBag getPropBag() {
	return _propertyContainer ;
    } // EndMethod getPropBag
    
    public void setName(String aNewName) {
	_name = aNewName ;
 	if (aNewName!=null) 
 	    if (aNewName.length()==0) aNewName = null ;

	if (aNewName!=null) setPropertyToValue("name",aNewName);
    } // EndMethod getName

    public String getName() {
	return _name ;
    } // EndMethod getName

    public int hashCode() {
        return _hashCode ;
    }

    /**
     * As {@link #equals(Object obj)} but does not throw exception in case
     * 2 equals links have not got the same properties.
     */
    public boolean equalsLazy(Object obj) {

        if (this.hashCode()!=obj.hashCode()) return false ;
	if (this==obj) return true ; // trivial case
	if (!(obj instanceof NamedEdge)) return false ;


	NamedEdge autreLink = (NamedEdge)obj ;
	
        // Optimization : we know equals has no side effect, so we can
        // optimize the sequence of comparison a little bit. The code
        // is equivalent to:

        // 	boolean result =
        // 	    this.getSink  ().equals(autreLink.getSink  () ) &&
        // 	    this.getSource().equals(autreLink.getSource() ) &&
        // 	    this.getName  ().equals(autreLink.getName  () ) ;
        
        // 	return result ;

        if (!this.getSink  ().equals(autreLink.getSink  () )) return false ;
        if (!this.getSource().equals(autreLink.getSource() )) return false ;
        if (!this.getName  ().equals(autreLink.getName  () )) return false ;

        return true ;
	
    } // EndMethod equalsLazy

    /**
     * 2 {@link NamedEdge } are equals if they have the same name, the same
     * source, and the same sink. An exception is thrown if 2 equals links
     * have not got the same properties.
     */
    public boolean equals(Object obj) {
// 	if (this==obj) return true ; // trivial case
// 	if (!(obj instanceof NamedEdge)) return false ;

	boolean result = equalsLazy(obj) ;

	// [CHANGE] ftaiani Mon Jul  2 2001 13:27:13
	// The equals method of NamedEdge now checks that the
	// properties of links are consistent with equality.
	// If the links are equals, but the property bags
	// are not, an exception is thrown.
	    
	NamedEdge autreLink = (NamedEdge)obj ;

	if ( result )
            if ( !this.getPropBag().equals( autreLink.getPropBag() ) )
                throw new RuntimeException
                    ("Conflicting link names: 2 links with same name, source, "+
                     "and sink, but with different properties."+ this + "\n" +
                     "Name  : " + getName() + "\n" +
                     "this  : " + this     .getPropBag() + "\n" + 
                     "autre : " + autreLink.getPropBag() 
                     ) ;

	return result ;

    } // EndMethod equals

    /**
     * The natural order of {@link NamedEdge } instance
     * is the lexical one base on the names of their sources,
     * followed by the names of their sinks, and finally
     * ended by their proper names.
     */
    public int compareTo(Object obj) {

	NamedEdge autreLink = (NamedEdge)obj ;
	int result ;
	result = ((NamedVertex)     this.getSource()).getName()    .compareTo
	       ( ((NamedVertex)autreLink.getSource()).getName() );

	if (result!=0) return result ;

	result = ((NamedVertex)     this.getSink()).getName()    .compareTo
	       ( ((NamedVertex)autreLink.getSink()).getName() );

	if (result!=0) return result ;

	return     this.getName()    .compareTo
	    ( autreLink.getName() ) ;

    } // EndMethod compareTo

    java.util.Set setOfPorts = new java.util.HashSet() ;

    /**
     * Is used in {@link XMLGraphBuilder} to label the edges
     * of caller / callee types with information on the order and nesting
     * of calls.
     * @see XMLGraphBuilder#addLinkFromXML( org.xml.sax.Attributes )
     * @see NamedVertex#countOfLinkChainings
     * @see #connectToIncomingPort(String port)
     */
    public boolean isAlreadyConnectedToIncomingPort(String port) {
	return setOfPorts.contains(port) ;
    } // EndMethod isAlreadyconnectedToIncomingPort

    /**
     * Is used in {@link XMLGraphBuilder} to label the edges
     * of caller / callee types with information on the order and nesting
     * of calls.
     * @see XMLGraphBuilder#addLinkFromXML( org.xml.sax.Attributes )
     * @see NamedVertex#countOfLinkChainings
     * @see #isAlreadyConnectedToIncomingPort(String port)
     */
    public void connectToIncomingPort(String port) {
	setOfPorts.add(port) ;
    } // EndMethod connectToIncomingPort

    /**
     * returns a property value of a the {@link PropertyBag} of the
     * <code>NamedVertex</code>.
     */
    public String getValueForProperty(String aPropertyName) {
	return _propertyContainer.getValueForProperty(aPropertyName) ;
    }

    public void setPropertyToValue(String aProperty, String aValue) {
	_propertyContainer.setPropertyToValue(aProperty,aValue) ;
    } ;

   /**
     * merges the attributes of 2 link.
     */
    public void addAttributesFromLink(NamedEdge link2) {

	PropertyBag propertieS1 = this .getPropBag() ;
	PropertyBag propertieS2 = link2.getPropBag() ;

	java.util.Iterator iter = propertieS2.iterator() ;

	while (iter.hasNext()) {
	    String currentProperty = (String)iter.next() ;

		propertieS1.changePropertyToValue
		    (currentProperty,
		     propertieS2.getValueForProperty(currentProperty) ) ;

	} // End While

    } //EndMethod addAttributesFromLink

} // EndClass NamedEdge
