/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * PropertyRenderer.java
 * 
 * Time-stamp: <2002-08-27 12:36:32 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: PropertyRenderer.java,v 1.3 2003/10/13 07:25:43 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis;

import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * PropertyRenderer's are used in 
 * {@link DirectedGraphWithDict2DotOStream#writeEdgeRendering(NamedEdge)}
 * to render dynamically new properties. To used a customized 
 * renderer you have to subclass this class, and defined 
 * {@link PropertyRenderer#formatProperty(String,PropertyBag,NamedEdge,DirectedGraphWithDict) formatProperty}.
 * @see DirectedGraphWithDict2DotOStream#registerRendererForKindOfLink(PropertyRenderer,String)
 * 
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.1.0
 */
public interface PropertyRenderer {
    /** 
     * Defines the "dot" formatting for the type of links the renderer is
     * registered with.
     * This method must be defined in any concrete renderer.
     * It will be called in 
     * {@link DirectedGraphWithDict2DotOStream#writeEdgeRendering(NamedEdge)}
     * each time a property is found in the link with name given
     * in the renderer constructor.
     * @param  kindOfLink        the kind of link the renderer is called for
     * @param  theLinkProperties the whole property bag of the processed link.
     * @param  linkToBeRendered  the actual link to be rendered.
     * @return the dot formating for that property.
     */
    public abstract String formatProperty(String        kindOfLink,
					  PropertyBag   theLinkProperties,
					  NamedEdge     linkToBeRendered,
					  DirectedGraphWithDict contextGraph
					  ) ;
} // EndClass PropertyRenderer
