/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * BagBasedTest.java
 * 
 * Time-stamp: <2003-03-10 13:09:40 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: BagBasedTest.java,v 1.3 2003/10/13 07:25:43 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * Base class for test cases in the package 
 * {@link laas.friends.cosmopen.graphAnalysis.test}. Because
 * individual tests may change the default comparison
 * behavior of {@link PropertyBag}, this base test
 * class insures the comparison behavior returns back
 * to normal before and after an individual test.
 * @author Francois Taiani   <taiani@laas.fr>
 */
public abstract class BagBasedTest extends junit.framework.TestCase {

    public BagBasedTest(String name) {
	super(name) ;
    } // EndConstructor BagBasedTest

    /** ****************************************************
     * Restores the way {@link PropertyBag}s are compared.
     * Launches at the beginning of each test.
     * The tests of the package {@link laas.friends.cosmopen.graphAnalysis.test}
     * may change the default way to compare {@link PropertyBag}s. This
     * ensure the situation is the default one at the beginning
     * of a test.
     * @see PropertyBag#DebugSetPropertiesRelevantForEquality(String[])
     */
    protected void setUp() throws java.lang.Exception {
	super.setUp() ;
	PropertyBag.DebugSetPropertiesRelevantForEquality(null) ;
    } // EndMethod setUp

    /** ****************************************************
     * Restores the way {@link PropertyBag}s are compared.
     * Launches at the end of each test.
     * The tests of the package {@link laas.friends.cosmopen.graphAnalysis.test}
     * may change the default way to compare {@link PropertyBag}s.  This
     * ensure the situation is the default one at the end
     * of a test.
     * @see PropertyBag#DebugSetPropertiesRelevantForEquality(String[])
     */
    protected void tearDown() throws java.lang.Exception {
	PropertyBag.DebugSetPropertiesRelevantForEquality(null) ;
	super.tearDown() ;
    }

} // EndClass BagBasedTest
