/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * GraphTesteeCreator.java
 * 
 * Time-stamp: <02/08/16 11:49:18 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: GraphTesteeCreator.java,v 1.3 2004/09/09 15:31:01 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;

import java.io.ByteArrayInputStream ;

/**
 * Very small to conveniently create graph testee from
 * string descriptions used for test. Also contains a default
 * graph testee.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.3 $
 * @since Version 0.0.4
 */
public class GraphTesteeCreator {

    public GraphTesteeCreator() {
    } // EndConstructor GraphTesteeCreator

    private static String graph1String =
	"<project>" +
	
	"  <class name=\"Arbre\">"           +
	"    <link target = \"BArbreCorde\" />" +
	"    <link target = \"CordeArbre\" />" +
	"  </class> " +

	"  <class name=\"BArbreCorde\">"           +
	"    <link target = \"Corde\" />" +
	"    <link target = \"CordeArbre\" />" +
	"  </class> " +

	"  <class name=\"CordeArbre\">"           +
	"    <link target = \"CordeArbre\" />" +
	"    <link target = \"Corde\" />" +
	"  </class> " +

	"  <class name=\"Corde\">"           +
	"    <link target = \"CordeArbre\" />" +
	"  </class> " +

	"</project>" ;

    /****************************************************************
     * create an exemplary graph to be used in some test methods.
     */
    public DirectedGraphWithDict createGraphTestee() 
	throws Exception 
    {
	return createGraphTestee(graph1String) ;
    } // EndMethod createGraphTestee

    /****************************************************************
     * create an exemplary graph to be used in some test methods.
     */
    public DirectedGraphWithDict createGraphTestee(String inputString) 
	throws Exception 
    {
	XMLGraphReader xmlReader ;
	DirectedGraphWithDict     graphTestee ;

	xmlReader = new XMLGraphReader();

	graphTestee = xmlReader.readGraphFromXML
	    (new ByteArrayInputStream(inputString.getBytes())) ;

	return graphTestee ;
	
    } // EndMethod createGraphTestee

    /****************************************************************
     * create an exemplary graph within a
     * given context to be used in some test methods.
     */
    public DirectedGraphWithDict createGraphTestee
	(String inputString, DirectedGraphWithDict context) 
	throws Exception 
    {
	XMLGraphReader xmlReader = new XMLGraphReader();

	DirectedGraphWithDict graphTestee = new DirectedGraphWithDictImpl() ;
	xmlReader.addGraphFromXMLToGraphInContext
	    ( graphTestee, 
	      context , 
	      new ByteArrayInputStream(inputString.getBytes())
	      ) ;

	return graphTestee ;
	
    } // EndMethod createGraphTestee

} // EndClass GraphTesteeCreator
