/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestCallGraph.java
 * 
 * Time-stamp: <2003-03-10 13:10:19 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestCallGraph.java,v 1.9 2007/09/18 18:23:42 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.MiscUtils ;

import java.util.* ;
import java.io.ByteArrayInputStream    ;
import java.io.InputStream    ;

/** *************************************************************
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.9 $
 * @since Version 0.1.1
 */
public class TestCallGraph  
    extends BagBasedTest
{
    private GenericXMLReader graphReader = new GenericXMLReader();

    // **************************************************************
    public TestCallGraph(String name) {
	super(name) ;
    } // EndConstructor TestCallGraph

    // **************************************************************
    private CallGraph readGraphFromString ( String aGraphDescription  ) throws Exception {

	return (CallGraph)graphReader.readGraphFromXML
	    ( new ByteArrayInputStream (aGraphDescription.getBytes()) ) ;

    } // EndMethod readGraphFromString

    // **************************************************************
    private CallGraph getTestee() throws Exception {
	CallGraph testee ;

	testee = readGraphFromString(TestXMLTraceSet2CallGraphReader.aCallGraphString) ;

	return testee ;

    } // EndMethod getTestee

    // **************************************************************
    private CallGraph getTestee2() throws Exception {
	CallGraph testee ;

	testee = readGraphFromString(TestXMLTraceSet2CallGraphReader.aCallGraphString2) ;

	return testee ;

    } // EndMethod getTestee

    // **************************************************************
    public void testGetSymbolicIDForInvocationVertex() throws Exception {
	CallGraph testee = getTestee() ;
	testee.prepareForVisualisation() ;
	Iterator vertexIt = testee.getVerticesIteratorAccordingToEventID();
	while(vertexIt.hasNext()) {
	    NamedVertex node = (NamedVertex)vertexIt.next();
	    String symbolicID = testee.getSymbolicIDForInvocationVertex(node).toString();
	    assertEquals(node.getValueForProperty("eventOrder"),symbolicID);
	} // EndWhile
	testee.endOfVisualisation();

	testee.remove( testee.getNodeWithName("A::callOnA't1-0") ) ;
	testee.remove( testee.getNodeWithName("A::callOnA't2-2") ) ;
	testee.remove( testee.getNodeWithName("B::callOnB't2-3") ) ;

	testee.prepareForVisualisation() ;
	vertexIt = testee.getVerticesIteratorAccordingToEventID();
	while(vertexIt.hasNext()) {
	    NamedVertex node = (NamedVertex)vertexIt.next();
	    Integer symbolicID = testee.getSymbolicIDForInvocationVertex(node);
	    if (node.getName().equals("B::callOnB't1-1")) {
		assertEquals("0",symbolicID.toString());
	    } else {
		int symbolicIntValue = symbolicID.intValue();
		int globalIntValue   = Integer.parseInt(node.getValueForProperty("eventOrder"));
		assertEquals("Testing Node "+node.getName(),globalIntValue-3,symbolicIntValue);
	    } // EndIfElse
	} // EndWhile
	testee.endOfVisualisation();

    } // EndMethod

    // **************************************************************
    public void testGetEventSBetweenEventID() throws Exception {

	CallGraph testee = getTestee() ;
	List result ;

	NamedVertex event0 = testee.getNodeWithName("A::callOnA't1-0");
	NamedVertex event1 = testee.getNodeWithName("B::callOnB't1-1");
	NamedVertex event2 = testee.getNodeWithName("A::callOnA't2-2");

	List expected = new Vector() ;
	expected.add(event0) ;
	expected.add(event1) ;
	expected.add(event2) ;

	result = testee.getEventSBetweenEventID(-1,3) ;

 	assertEquals("-1,3",expected,result) ;

	NamedVertex event3 = testee.getNodeWithName("B::callOnB't2-3");
	NamedVertex event4 = testee.getNodeWithName("D::callOnD't1-4");
	NamedVertex event5 = testee.getNodeWithName("B::callOnB't1-5");

	expected = new Vector() ;
	expected.add(event3) ;
	expected.add(event4) ;
	expected.add(event5) ;

	result = testee.getEventSBetweenEventID(3,6) ;

 	assertEquals("3,6",expected,result) ;

	NamedVertex event9 = testee.getNodeWithName("C::callOnC't1-9");
	NamedVertex event10 = testee.getNodeWithName("C::callOnC't2-10");

	expected = new Vector() ;
	expected.add(event9) ;
	expected.add(event10) ;

	result = testee.getEventSBetweenEventID(9,-1) ;

 	assertEquals("9,-1",expected,result) ;

	result = testee.getEventSBetweenEventID(6,6) ;

 	assertEquals("6,6",new Vector(),result) ;

	result = testee.getEventSBetweenEventID(6,3) ;

 	assertEquals("6,3",new Vector(),result) ;

	result = testee.getEventSBetweenEventID(-1,-1) ;
	assertTrue("-1,-1",result.size()==11) ;

    } // EndMethod testGetEventSBetweenEventID

    // **************************************************************
    public void testAbstractAway() throws Exception {
	
	CallGraph reference = getTestee() ;
	NamedVertex a = reference.getNodeWithName("A::callOnA't1-0") ;
	CallGraph testee   = (CallGraph)reference.clone() ;
	CallGraph expected = (CallGraph)reference.clone() ;

	testee   .abstractAway(a) ;
	expected .remove      (a) ;
	assertTrue(expected.equalsDebug(testee)) ;

	// abstracting away again should not change anything

	testee   .abstractAway(a) ;
	assertTrue(expected.equalsDebug(testee)) ;

	testee   = (CallGraph)reference.clone() ;
	expected = (CallGraph)reference.clone() ;

	NamedVertex b = reference.getNodeWithName("B::callOnB't2-3");
	testee   .abstractAway(b) ;
	expected .remove      (b) ;

	String linkToAdd = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::callOnA't2-2\" entity=\"A\" " +
	    "         method=\"callOnA\" eventOrder=\"2\" threadID=\"2\" > " +
	    "    <link target = \"C::callOnC't2-10\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " +
	    "  <class name=\"C::callOnC't2-10\" entity=\"C\" " +
	    "         method=\"callOnC\"  eventOrder=\"10\" threadID=\"2\" kind=\"call\" > " +
	    "  </class> " + 
	    "</project>" ;
	
	expected.addGraph(readGraphFromString(linkToAdd)) ;
	assertTrue(expected.equalsDebug(testee)) ;

	testee = getTestee2() ;
	testee .abstractAway(testee.getNodeWithName("C::'t1-2")) ;

	String expectedAsString =  
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" method=\"\" >"+
	    "    <link target = \"B::'t1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "    <link target = \"D::'t2-3\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	    "    <link target = \"G::'t1-6\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'t1-1\" entity=\"B\" eventOrder=\"1\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "  <class name=\"D::'t2-3\" entity=\"D\" eventOrder=\"3\" threadID=\"2\" "+
	    "         kind=\"threadCreate\" method=\"\" > " +
	    "    <link target = \"B::'t2-4\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'t2-4\" entity=\"B\" eventOrder=\"4\" threadID=\"2\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "    <link target = \"F::'t2-5\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"F::'t2-5\" entity=\"F\" eventOrder=\"5\" threadID=\"2\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "  <class name=\"G::'t1-6\" entity=\"G\" eventOrder=\"6\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "</project>" ;

	assertTrue(readGraphFromString(expectedAsString).equalsDebug(testee)) ;

	String graphPatch =
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" method=\"\" >"+
	    "    <link target = \"C::'t1-2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"C::'t1-2\" entity=\"C\" eventOrder=\"2\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "    <link target = \"D::'t2-3\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	    "  </class> " + 
	    "  <class name=\"D::'t2-3\" entity=\"D\" eventOrder=\"3\" threadID=\"2\" "+
	    "         kind=\"threadCreate\" method=\"\" > " +
	    "  </class> " + 
	    "</project>" ;

	// Putting back the abstracted node with some of its child should undo the
	// abstraction. (Weird, but well.)

	testee.addGraph(readGraphFromString(graphPatch));
	assertTrue(getTestee2().equalsDebug(testee)) ;
	
	testee = getTestee2() ;
	testee   .abstractAway(testee.getNodeWithName("D::'t2-3")) ;

	expectedAsString = 
	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" eventOrder=\"0\" threadID=\"1\" method=\"\" >"+
	    "    <link target = \"B::'t1-1\" kind=\"controlflow\" subkind=\"call\" />" +
	    "    <link target = \"C::'t1-2\" kind=\"controlflow\" subkind=\"call\" />" +
	    "    <link target = \"G::'t1-6\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'t1-1\" entity=\"B\" eventOrder=\"1\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "  <class name=\"C::'t1-2\" entity=\"C\" eventOrder=\"2\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "    <link target = \"B::'t2-4\" kind=\"controlflow\" subkind=\"threadCreate\" />" +
	    "  </class> " + 
	    "  <class name=\"B::'t2-4\" entity=\"B\" eventOrder=\"4\" threadID=\"2\" "+
	    "         kind=\"threadCreate\" method=\"\" > " +
	    "    <link target = \"F::'t2-5\" kind=\"controlflow\" subkind=\"call\" />" +
	    "  </class> " + 
	    "  <class name=\"F::'t2-5\" entity=\"F\" eventOrder=\"5\" threadID=\"2\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "  <class name=\"G::'t1-6\" entity=\"G\" eventOrder=\"6\" threadID=\"1\" "+
	    "         kind=\"call\" method=\"\" > " +
	    "  </class> " + 
	    "</project>" ;

	assertTrue(readGraphFromString(expectedAsString).equalsDebug(testee)) ;
	
	String inputString =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "</traceSet>" ; 
	testee = 
	    XMLTraceSet2CallGraphReader.ReadCallGraphFromString(inputString) ;

	expectedAsString =
	    "<project type=\"callGraph\" >" +
	    "  <class eventOrder=\"0\" entity=\"A\" name=\"A::'t1-0\" "+
	    "         threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t1-4\" kind=\"controlflow\" "+
	    "          target=\"E::'t1-4\" />" +
	    "    <link subkind=\"call\" name=\"C::'t1-2\" kind=\"controlflow\" "+
	    "          target=\"C::'t1-3\" />" +
	    "    <link subkind=\"threadCreate\" name=\"D::'t2-3\" kind=\"controlflow\" "+
	    "          target=\"D::'t2-2\" />" +
	    "  </class>" +
	    "  <class eventOrder=\"3\" entity=\"C\" name=\"C::'t1-3\" kind=\"call\" "+
	    "         threadID=\"1\" method=\"\" >" +
	    "  </class>" +
	    "  <class eventOrder=\"2\" entity=\"D\" name=\"D::'t2-2\" kind=\"threadCreate\" "+
	    "         threadID=\"2\" method=\"\" >" +
	    "  </class>" +
	    "  <class eventOrder=\"4\" entity=\"E\" name=\"E::'t1-4\" kind=\"call\" "+
	    "         threadID=\"1\" method=\"\" >" +
	    "  </class>" +
	    "</project>" ;

	testee.abstractAway(testee.getNodeWithName("B::'t1-1")) ;
	assertTrue(readGraphFromString(expectedAsString).equalsDebug(testee)) ;

    } // EndMethod testAbstractAway

    public String leapOverTestTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ; 


    public String leapOverExpectedAsString =
	    "<project type=\"callGraph\" >" +
	    "<class eventOrder=\"0\" entity=\"A\" name=\"A::'t1-0\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-1\" "+
	    "     kind=\"controlflow\" target=\"C::'t1-1\" />" +
	    "</class>" +
	    "<class eventOrder=\"1\" entity=\"C\" name=\"C::'t1-1\" "+
	    "       kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"F::'t2-4\" "+
	    "     kind=\"controlflow\" target=\"F::'t2-4\" />" +
	    "</class>" +
	    "<class eventOrder=\"2\" entity=\"D\" name=\"D::'t2-2\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-3\" "+
	    "     kind=\"controlflow\" target=\"E::'t2-3\" />" +
	    "</class>" +
	    "<class eventOrder=\"3\" entity=\"E\" name=\"E::'t2-3\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"4\" entity=\"F\" name=\"F::'t2-4\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"5\" entity=\"A2\" name=\"A2::'t1-5\" "+
	    "       threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-6\" "+
	    "     kind=\"controlflow\" target=\"C::'t1-6\" />" +
	    "</class>" +
	    "<class eventOrder=\"6\" entity=\"C\" name=\"C::'t1-6\""+
	    "       kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"threadCreate\" name=\"F::'t3-9\" "+
	    "     kind=\"controlflow\" target=\"F::'t3-9\" />" +
	    "</class>" +
	    "<class eventOrder=\"7\" entity=\"D2\" name=\"D2::'t2-7\" "+
	    "       threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-8\" "+
	    "     kind=\"controlflow\" target=\"E::'t2-8\" />" +
	    "</class>" +
	    "<class eventOrder=\"8\" entity=\"E\" name=\"E::'t2-8\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"9\" entity=\"F\" name=\"F::'t3-9\""+
	    "       kind=\"threadCreate\" threadID=\"3\" method=\"\" >" +
	    "</class>" +
	    "</project>" ;

    // **************************************************************
    public void testLeapOverNodes() throws Exception {

	String expectedAsString =
	    "<project type=\"callGraph\" >" +
	    "<class eventOrder=\"0\" entity=\"A\" name=\"A::'t1-0\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-1\" "+
	    "     kind=\"controlflow\" target=\"C::'t1-1\" />" +
	    "</class>" +
	    "<class eventOrder=\"1\" entity=\"C\" name=\"C::'t1-1\" "+
	    "       kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"2\" entity=\"D\" name=\"D::'t2-2\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-3\" "+
	    "     kind=\"controlflow\" target=\"E::'t2-3\" />" +
	    "</class>" +
	    "<class eventOrder=\"3\" entity=\"E\" name=\"E::'t2-3\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"F::'t2-4\" "+
	    "     kind=\"controlflow\" target=\"F::'t2-4\" />" +
	    "</class>" +
	    "<class eventOrder=\"4\" entity=\"F\" name=\"F::'t2-4\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"5\" entity=\"A2\" name=\"A2::'t1-5\" "+
	    "       threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-6\" "+
	    "     kind=\"controlflow\" target=\"C::'t1-6\" />" +
	    "</class>" +
	    "<class eventOrder=\"6\" entity=\"C\" name=\"C::'t1-6\""+
	    "       kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"7\" entity=\"D2\" name=\"D2::'t2-7\" "+
	    "       threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-8\" "+
	    "     kind=\"controlflow\" target=\"E::'t2-8\" />" +
	    "</class>" +
	    "<class eventOrder=\"8\" entity=\"E\" name=\"E::'t2-8\""+
	    "       kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"threadCreate\" name=\"F::'t3-9\" "+
	    "     kind=\"controlflow\" target=\"F::'t3-9\" />" +
	    "</class>" +
	    "<class eventOrder=\"9\" entity=\"F\" name=\"F::'t3-9\""+
	    "       kind=\"threadCreate\" threadID=\"3\" method=\"\" >" +
	    "</class>" +
	    "</project>" ;

	CallGraph testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(leapOverTestTrace);
	// If this does not work, the test won't work for sure.
	assertTrue("Test consistency",readGraphFromString(expectedAsString).equalsDebug(testee)) ;

	testee.leapOverNodes( testee.getNodeSWithRegExp("C::*"),
			      testee.getNodeSWithRegExp("E::*" ) ) ;


	assertTrue("After leapOver",readGraphFromString(leapOverExpectedAsString).equalsDebug(testee)) ;

    } // EndMethod testLeapOverNodes

    // **************************************************************
    public void testRebind() throws Exception {

	CallGraph testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(leapOverTestTrace);

 	testee.rebind( testee.getNodeSWithRegExp("C::*"),
 		       testee.getNodeSWithRegExp("F::*" ) ) ;


	assertTrue("After rebind",readGraphFromString(leapOverExpectedAsString).equalsDebug(testee)) ;

	String problemTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ; 

	testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);

	try{

	testee.rebind( testee.getNodeSWithRegExp("C::*"),
		       testee.getNodeSWithRegExp("F::*" ) ) ;
	} catch (Exception e) {
	    CallGraph sameAsTestee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);
	    // If rebind fails no change should be made to the graph
	    assertTrue("After failed rebind",sameAsTestee.equalsDebug(testee)) ;
	    return ;
	}
	
	fail("An ambiguous trace should raise an exception.");

    } // EndMethod testRebind

    // **************************************************************
    public void testFuse() throws Exception {

	CallGraph testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(leapOverTestTrace);

 	testee.fuse( testee.getNodeSWithRegExp("C::*"),
			   testee.getNodeSWithRegExp("F::*" ) ) ;


	assertTrue("After rebind",readGraphFromString(leapOverExpectedAsString).equalsDebug(testee)) ;

        String fuseTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ; 

        String fuseExpectedAsString =
	    "<project type=\"callGraph\" >" +
	    "<class eventOrder=\"0\" entity=\"A\" name=\"A::'t1-0\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-1\" kind=\"controlflow\" target=\"C::'t1-1\" />" +
	    "    <link subkind=\"call\" name=\"F::'t2-6\" kind=\"controlflow\" target=\"F::'t2-6\" />" +
	    "</class>" +
	    "<class eventOrder=\"1\" entity=\"C\" name=\"C::'t1-1\" kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"2\" entity=\"A2\" name=\"A2::'t1-2\" threadID=\"1\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"C::'t1-3\" kind=\"controlflow\" target=\"C::'t1-3\" />" +
	    "    <link subkind=\"threadCreate\" name=\"F::'t3-9\" kind=\"controlflow\" target=\"F::'t3-9\" />" +
	    "</class>" +
	    "<class eventOrder=\"3\" entity=\"C\" name=\"C::'t1-3\" kind=\"call\" threadID=\"1\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"4\" entity=\"D\" name=\"D::'t2-4\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-5\" kind=\"controlflow\" target=\"E::'t2-5\" />" +
	    "</class>" +
	    "<class eventOrder=\"5\" entity=\"E\" name=\"E::'t2-5\" kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"6\" entity=\"F\" name=\"F::'t2-6\" kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"7\" entity=\"D2\" name=\"D2::'t2-7\" threadID=\"2\" method=\"\" >" +
	    "    <link subkind=\"call\" name=\"E::'t2-8\" kind=\"controlflow\" target=\"E::'t2-8\" />" +
	    "</class>" +
	    "<class eventOrder=\"8\" entity=\"E\" name=\"E::'t2-8\" kind=\"call\" threadID=\"2\" method=\"\" >" +
	    "</class>" +
	    "<class eventOrder=\"9\" entity=\"F\" name=\"F::'t3-9\" kind=\"threadCreate\" threadID=\"3\" method=\"\" >" +
	    "</class>" +
	    "</project>"
	    ;	

	testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(fuseTrace);

 	testee.fuse( testee.getNodeSWithRegExp("A*"),
			   testee.getNodeSWithRegExp("F::*" ) ) ;


	assertTrue("After rebind",readGraphFromString(fuseExpectedAsString).equalsDebug(testee)) ;


    } // EndMethod testFuse

    public void testFuseNotEnoughChildren() throws Exception {

	String problemTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D2\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	CallGraph testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);

	try{

	testee.fuse( testee.getNodeSWithRegExp("A*"),
		     testee.getNodeSWithRegExp("F::*" ) ) ;
	} catch (Exception e) {
	    CallGraph sameAsTestee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);
	    // If rebind fails no change should be made to the graph
	    assertTrue("After failed fuse",sameAsTestee.equalsDebug(testee)) ;
	    return ;
	}
	
	fail("An ambiguous trace should raise an exception.");

    } // EndMethod testFuseNotEnoughChildren

    public void testFuseParentYoungerThanChild() throws Exception {

	String problemTrace =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "    <call entity=\"D2\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A2\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

	CallGraph testee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);

	try{

	testee.fuse( testee.getNodeSWithRegExp("A*"),
		     testee.getNodeSWithRegExp("F::*" ) ) ;
	} catch (Exception e) {
	    CallGraph sameAsTestee = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemTrace);
	    // If rebind fails no change should be made to the graph
	    assertTrue("After failed fuse",sameAsTestee.equalsDebug(testee)) ;
	    return ;
	}
	
	fail("An ambiguous trace should raise an exception.");

    } // EndMethod testFuseNotEnoughChildren


    // **************************************************************
    public void testGetMostRecentEventBeforeID() throws Exception {
	CallGraph testee   = getTestee() ;
	NamedVertex event2 = testee.getNodeWithName("A::callOnA't2-2");

	assertEquals("getMostRecentEventBeforeID on existing event does not work.",
		     event2,testee.getMostRecentEventBeforeID(2)) ;

	NamedVertex event3  = testee.getNodeWithName("B::callOnB't2-3");
	testee.remove(event3) ;
	
	assertEquals("getMostRecentEventBeforeID with 1 missing event does not work.",
		     event2,testee.getMostRecentEventBeforeID(2)) ;

	NamedVertex event4  = testee.getNodeWithName("D::callOnD't1-4");
	testee.remove(event4) ;

	assertEquals("getMostRecentEventBeforeID with 2 missing event does not work.",
		     event2,testee.getMostRecentEventBeforeID(4)) ;

	assertEquals("getMostRecentEventBeforeID on not existing event does not work.",
		     null,testee.getMostRecentEventBeforeID(-1)) ;
    } // EndMethod testGetMostRecentEventBeforeID

    final static String graphWithRecursiveCalls =
	"<traceSet>" +
	"  <trace thread=\"1\" > " +
	"    <call entity=\"B\" method=\"callOnB\" />" +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"  </trace> " +
	"  <trace thread=\"2\" > " +
	"    <call entity=\"B\" method=\"callOnB\" />" +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"  </trace> " +
	"  <trace thread=\"1\" > " +
	"    <call entity=\"C\" method=\"callOnC\" />" +
	"    <call entity=\"B\" method=\"callOnB\" />" +
	"    <call entity=\"D\" method=\"callOnD\" />" +
	"  </trace> " +
	"  <trace thread=\"1\"> " +
	"    <call entity=\"C\" method=\"callOnC\" />" +
	"    <call entity=\"B\" method=\"callOnB\" />" +
	"    <call entity=\"B\" method=\"callOnB2\" />" +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"  </trace> " +
	"  <trace thread=\"2\"> " +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"    <call entity=\"\" method=\"noEntity4\" />" +
	"    <call entity=\"\" method=\"noEntity3\" />" +
	"    <call entity=\"\" method=\"noEntity2\" />" +
	"    <call entity=\"\" method=\"noEntity1\" />" +
	"    <call entity=\"C\" method=\"callOnC2\" />" +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"    <call entity=\"\" method=\"noEntity1\" />" +
	"    <call entity=\"\" method=\"noEntity1\" />" +
	"    <call entity=\"\" method=\"noEntity1\" />" +
	"    <call entity=\"C\" method=\"callOnC\" />" +
	"    <call entity=\"C\" method=\"callOnC2\" />" +
	"    <call entity=\"B\" method=\"callOnB\" />" +
	"    <call entity=\"A\" method=\"callOnA\" />" +
	"  </trace> " +
	"</traceSet>" ;	

    // **************************************************************
    public void testRemoveSelfCallOnNodeMatchingRegexp() throws Exception {
	CallGraph backup   = readGraphFromString(graphWithRecursiveCalls) ;
	CallGraph testee   ;
	CallGraph expected ;

	testee   = (CallGraph)backup.clone() ;
	expected = (CallGraph)backup.clone() ;
	testee.removeSelfCallOnNodeMatchingRegexp("B::*") ;
	NamedVertex event8 = backup.getNodeWithName("B::callOnB't1-9") ;
	expected.abstractAway(event8) ;
	assertEquals("Cleaning B::*",expected,testee) ;
	
	testee   = (CallGraph)backup.clone() ;
	expected = (CallGraph)backup.clone() ;
	testee.removeSelfCallOnNodeMatchingRegexp("C::*") ;
	NamedVertex event11 = backup.getNodeWithName("C::callOnC't2-12") ;
	expected.abstractAway(event11) ;
	assertEquals("Cleaning C::*",expected,testee) ;
	
	// TESTING ON CALL OF PLAIN FUNCTION (entity is void)
	// In this particular case, we expect that only calls
	// recusrive on the same function are removed

	testee   = (CallGraph)backup.clone() ;
	expected = (CallGraph)backup.clone() ;
	testee.removeSelfCallOnNodeMatchingRegexp("::noEntity*") ;
	NamedVertex event13 = backup.getNodeWithName("::noEntity1't2-14") ;
	NamedVertex event14 = backup.getNodeWithName("::noEntity1't2-15") ;
	expected.abstractAway(event13) ;
	expected.abstractAway(event14) ;
	assertEquals("Cleaning ::noEntity*",expected,testee) ;

	testee   = (CallGraph)backup.clone() ;
	expected = (CallGraph)backup.clone() ;
	testee.removeSelfCallOnNodeMatchingRegexp("*") ;
	expected.abstractAway(event8) ;
	expected.abstractAway(event11) ;
	expected.abstractAway(event13) ;
	expected.abstractAway(event14) ;
	assertEquals("Cleaning *",expected,testee) ;
	
    } // EndMethod testRemoveSelfCallOnNodeMatchingRegexp

    // **************************************************************
    public void testShortEntityName() {
	assertEquals(MiscUtils.GetShortEntityName(""),"");
	assertEquals(MiscUtils.GetShortEntityName("AnEntity"),"AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("namespace::AnEntity"),"AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("namespace2::namespace1::AnEntity"),
		     "AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("AnEntity<>"),"AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("AnEntity<someParameter>"),"AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("AnEntity<param1,param2<param3>>"),"AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("namespace::AnEntity<param1,param2<param3>>"),
		     "AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("N1::N2::AnEntity<param1,param2<param3>>"),
		     "AnEntity");
	assertEquals(MiscUtils.GetShortEntityName("OB::stl::list<OB::ObjVar<OCI::CloseCB> >"),
		     "list");
    } // EndM

} // EndClass TestCallGraph
