/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestCallGraphToCondensedGraphTransformer.java
 * 
 * Time-stamp: <2003-03-11 10:52:04 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestCallGraphToCondensedGraphTransformer.java,v 1.8 2007/09/18 18:23:42 ftaiani_normal Exp $
 * 
 */

package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.PropertyBag ;

import java.io.ByteArrayInputStream    ;
import java.io.InputStream    ;
import java.util.* ;

/**
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.8 $
 * @since Version 0.1.1
 */
public class TestCallGraphToCondensedGraphTransformer 
    extends BagBasedTest {

    private 
	CallGraphToCondensedGraphTransformer 
	condenserTestee = new CallGraphToCondensedGraphTransformer() ;
    private 
	GenericXMLReader
	graphReader     = new GenericXMLReader() ; // XMLGraphReader();

    protected void tearDown() throws java.lang.Exception {
	CallGraphToCondensedGraphTransformer.setExplicitRange(false) ;
	super.tearDown() ;
    }

    /** *********************************************/
    public TestCallGraphToCondensedGraphTransformer(String name) {
	super(name) ;
    } // EndConstructor TestCallGraphToCondensedGraphTransformer

    /** *********************************************
     *  Helper class. It checks that the
     *  graph obtained after converting <code>aTraceSetString</code>
     *  into a normal graph is equal to the graph passed in xml 
     *  format through <code>aGraphString</code>.
     */
    private void testEqualityAfterTransformFromTrace( String startTraceSetString, 
						      String expectedResult    ) 
	throws Exception 
    {
 	CallGraph tempCallgraph 
	    = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(startTraceSetString) ;
	testEqualityAfterTransformFromGraph(tempCallgraph,expectedResult);
    }

    private void testEqualityAfterTransformFromString( String startCallGraphString, 
						       String expectedResult    ) 
	throws Exception 
    {
 	CallGraph tempCallgraph 
	    = (CallGraph)graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(startCallGraphString.getBytes()) ) ;
	testEqualityAfterTransformFromGraph(tempCallgraph,expectedResult);
    }

    private void testEqualityAfterTransformFromGraph(CallGraph startCallGraph,
						     String    expectedResult)
	throws Exception 
    {
	
	DirectedGraphWithDict result ;
	result = condenserTestee.fromCallGraphToGraph(startCallGraph);

	DirectedGraphWithDict expectedGraph ;
	expectedGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expectedResult.getBytes()) ) ;

//     	System.out.println("\nEXPECTED\n\n" + expectedGraph ) ;
//     	System.out.println("\nRESULT  \n\n" + result ) ;

	assertTrue( expectedGraph.equalsDebug(result) ) ;

    } // EndMethod testEqualityAfterTransformFromGraph

    /** *********************************************/
    public void testCallGraphConvertion() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{ "isStart", "callOrder", "name", 
		  "kind"   , "target"   , "incomingOrder", "method" } ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"C\" method=\"mC2\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project >" +
	    "  <class name=\"A\" isRoot=\"y\" > " +
	    "    <link target = \"B\" name =\"mB1\" kind=\"call\" callOrder=\"1\" "+
	    "          incomingOrder=\"a\" "+
	    "          method=\"mB1\" isStart=\"y\""/*insert*/+" />" +
	    "    <link target = \"C\" name =\"mC2\" kind=\"call\" callOrder=\"2\" "+
	    "          incomingOrder=\"b\" "+
	    "          method=\"mC2\" isReturn=\"y\""/*insert*/+" />" +
	    "  </class> " + 
	    "  <class name=\"B\" > " +
	    "    <link target = \"C\" name =\"mC1\" kind=\"call\" incomingOrder=\"a\" "+
	    "          method=\"mC1\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"a\" isStart=\"y\" isReturn=\"y\" />" +
	    "  </class> " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"B\" name =\"mB2\" kind=\"call\" incomingOrder=\"b\" "+
	    "          method=\"mB2\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"b\" isStart=\"y\" isReturn=\"y\" />" +
	    "  </class> " + 
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testCallGraphConstruction

    /** *********************************************/
    public void testCallGraphConvertionWithFunctions() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{ "isStart", "callOrder", "name", 
		  "kind"   , "target"   , "incomingOrder", 
		  "method" , "isFunction" } ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"\"  method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"\"  method=\"mB1\" />" +
	    "    <call entity=\"C\" method=\"mC2\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project >" +
	    "  <class name=\"A\" isRoot=\"y\" > " +
	    "    <link target = \"mB1\" name =\"mB1\" kind=\"call\" callOrder=\"1\" "+
	    "          incomingOrder=\"a\" "+
	    "          method=\"mB1\" isStart=\"y\""/*insert*/+" />" +
	    "    <link target = \"C\" name =\"mC2\" kind=\"call\" callOrder=\"2\" "+
	    "          incomingOrder=\"b\" "+
	    "          method=\"mC2\" isReturn=\"y\""/*insert*/+" />" +
	    "  </class> " + 
	    "  <class name=\"mB1\" isFunction=\"y\" > " +
	    "    <link target = \"C\" name =\"mC1\" kind=\"call\" incomingOrder=\"a\" "+
	    "          method=\"mC1\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"a\" isStart=\"y\" isReturn=\"y\" />" +
	    "  </class> " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"mB1\" name =\"mB1\" kind=\"call\" incomingOrder=\"b\" "+
	    "          method=\"mB1\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"b\" isStart=\"y\" isReturn=\"y\" />" +
	    "  </class> " + 
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testCallGraphConstruction

    /** *********************************************/
    public void testEventNumbering() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{   "name",
 		    "globalOrder",
 		    "target",
		    "name"
		} ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"C\" method=\"mC2\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project >" +
	    "  <class name=\"A\" > " +
	    "    <link target = \"B\" name =\"mB1\" globalOrder=\"1\" />" +
	    "    <link target = \"C\" name =\"mC2\" globalOrder=\"3\" />" +
	    "  </class>            " + 
	    "  <class name=\"B\" > " +
	    "    <link target = \"C\" name =\"mC1\" globalOrder=\"2\" />" +
	    "  </class>            " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"B\" name =\"mB2\" globalOrder=\"4\" />" +
	    "  </class>            " + 
	    "</project>" ;

 	CallGraph theCallgraph ;

	DirectedGraphWithDict result ;
	DirectedGraphWithDict expectedGraph ;

 	theCallgraph  = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(input) ;

	result        = condenserTestee.fromCallGraphToGraph(theCallgraph) ;
	expectedGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expected.getBytes()) ) ;

//     	System.out.println("\nEXPECTED\n\n" + expectedGraph ) ;
//     	System.out.println("\nRESULT  \n\n" + result ) ;

	assertTrue( expectedGraph.equalsDebug(result) ) ;

	String expected2 =
	    "<project >" +
	    "  <class name=\"B\" > " +
	    "    <link target = \"C\" name =\"mC1\" globalOrder=\"1\" />" +
	    "  </class>            " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"B\" name =\"mB2\" globalOrder=\"2\" />" +
	    "  </class>            " + 
	    "</project>" ;

	theCallgraph.remove(theCallgraph.getNodeWithName("A::mA1'0")) ;
	result        = condenserTestee.fromCallGraphToGraph(theCallgraph) ;
	expectedGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expected2.getBytes()) ) ;
	
	assertTrue( expectedGraph.equalsDebug(result) ) ;

	String expected3 =
	    "<project >" +
	    "  <class name=\"A\" > " +
	    "    <link target = \"B\" name =\"mB1\" globalOrder=\"1\" />" +
	    "    <link target = \"C\" name =\"mC2\" globalOrder=\"2\" />" +
	    "  </class>            " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"B\" name =\"mB2\" globalOrder=\"3\" />" +
	    "  </class>            " + 
	    "</project>" ;

 	theCallgraph  = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(input) ;
	theCallgraph.remove(theCallgraph.getNodeWithName("C::mC1'2")) ;

	result        = condenserTestee.fromCallGraphToGraph(theCallgraph) ;
	expectedGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expected3.getBytes()) ) ;
	
	assertTrue( expectedGraph.equalsDebug(result) ) ;


    } // EndMethod testEventNumbering

    /** *********************************************/
    public void testCallGraphConvertionWithThread() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{ "method", "isStart", "name", "incomingOrder", "callOrder", 
		  "kind", "target", "thread", "newThread", "precedent" } ) ;

	String input =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" + 
	    "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"C\" method=\"mC2\" />" +
	    "    <call entity=\"A\" method=\"mA2\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project >" +
	    "  <class name=\"A\" isRoot=\"y\" > " +
	    "    <link target = \"B\" name =\"1:mB1\" kind=\"call\" callOrder=\"1\" "+
	    "          incomingOrder=\"a\" "+
	    "          method=\"mB1\" thread=\"1\""/*insert*/+" "+
	    "          precedent=\"a\" isReturn=\"y\" isStart=\"y\"/>" +
	    "    <link target = \"C\" name =\"2:mC2\" kind=\"call\" callOrder=\"2\" "+
	    "          incomingOrder=\"b\" "+
	    "          method=\"mC2\" thread=\"2\""/*insert*/+" "+
	    "          precedent=\"b\" isReturn=\"y\" isStart=\"y\"/>" +
	    "  </class> " + 
	    "  <class name=\"B\" > " +
	    "    <link target = \"A\" name =\"1N2:mA2\" kind=\"threadCreate\" "+
	    "          callOrder=\"1\" incomingOrder=\"b\" "+ 
	    "          method=\"mA2\" thread=\"1\" newThread=\"2\" "/*insert*/+" "+
	    "          precedent=\"a\" isStart=\"y\" />" +
	    "    <link target = \"C\" name =\"1:mC1\" kind=\"call\" callOrder=\"2\" "+
	    "          incomingOrder=\"a\" "+
	    "          method=\"mC1\" thread=\"1\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"a\" isReturn=\"y\" />" +
	    "  </class> " + 
	    "  <class name=\"C\" > " +
	    "    <link target = \"B\" name =\"2:mB2\" kind=\"call\" incomingOrder=\"b\" "+ 
	    "          method=\"mB2\" thread=\"2\" isLeaf=\"y\""/*insert*/+" "+
	    "          precedent=\"b\" isReturn=\"y\" isStart=\"y\" />" +
	    "  </class> " + 
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testCallGraphConvertionWithThread

    public void testCallGraphConvertionWithThreadWicked() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{ "method", "isStart", "name", "incomingOrder", "callOrder", 
		  "kind", "target", "thread", "newThread", "precedent" } ) ;

	String input =
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"A\" method=\"mA\" />" + 
	    "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"mB\" />" + 
	    "    <call entity=\"A\" method=\"mA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"B\" method=\"mB\" />" +
	    "    <call entity=\"A\" method=\"mA\" />" +
	    "    <call entity=\"B\" method=\"mB\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project >" +
	    "  <class name=\"A\" isRoot=\"y\" > " +
	    "    <link target = \"B\" name =\"1N2:mB\" kind=\"threadCreate\" callOrder=\"1\" "+
	    "          incomingOrder=\"a\" "+
	    "          method=\"mB\" thread=\"1\" newThread=\"2\" "/*insert*/+" "+
	    "          precedent=\"a\" isStart=\"y\"/>" +
	    "    <link target = \"B\" name =\"1:mB\" kind=\"call\" callOrder=\"2\" "+
	    "          incomingOrder=\"b\" "+
	    "          method=\"mB\" thread=\"1\" "/*insert*/+" "+
	    "          precedent=\"a\" isReturn=\"y\" isLeaf=\"y\"/>" +
	    "    <link target = \"B\" name =\"2:mB\" kind=\"call\" callOrder=\"3\" "+
	    "          incomingOrder=\"c\" "+
	    "          method=\"mB\" thread=\"2\" "/*insert*/+" "+
	    "          precedent=\"b\" isReturn=\"y\" isStart=\"y\" isLeaf=\"y\" />" +
	    "  </class> " + 
	    "  <class name=\"B\" > " +
	    "    <link target = \"A\" name =\"2:mA\" kind=\"call\" "+
	    "          incomingOrder=\"b\" "+ 
	    "          method=\"mA\" thread=\"2\" "/*insert*/+" "+
	    "          precedent=\"a\" isStart=\"y\" isReturn=\"y\"/>" +
	    "  </class> " + 
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testCallGraphConvertionWithThreadWicked

    public void testReturnTag() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] { "isReturn", "isStart", "name", "kind", 
			      "target", "incomingOrder", "method" } ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"mE1\" />" +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"D\" method=\"mD1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"F\" method=\"mE1\" />" +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"D\" method=\"mD1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "  </trace> " +
	    "</traceSet>" ;

	String expected =
	    "<project>" +
	    "<class name=\"B\" isRoot=\"y\" >" +
	    "    <link callOrder=\"1\" method=\"mC1\" name=\"mC1\" kind=\"call\" "+
	    "           precedent=\"a\" target=\"C\" isStart=\"y\" isLeaf=\"y\"/>" +
	    "    <link callOrder=\"2\" method=\"mD1\" name=\"mD1\" kind=\"call\" "+
	    "           precedent=\"a\" target=\"D\" isReturn=\"y\" />" +
	    "    <link callOrder=\"3\" method=\"mE1\" name=\"mE1\" kind=\"call\" "+
	    "           precedent=\"b\" target=\"E\" isStart=\"y\" isLeaf=\"y\"/>" +
	    "    <link callOrder=\"4\" method=\"mE1\" name=\"mE1\" kind=\"call\" "+
	    "           precedent=\"b\" target=\"F\" isReturn=\"y\" isLeaf=\"y\" />" +
	    "</class>" +
	    "<class name=\"C\" >" +
	    "</class>" +
	    "<class name=\"D\" >" +
	    "    <link method=\"mB2\" name=\"mB2\" kind=\"call\" target=\"B\" "+
	    "           incomingOrder=\"b\" isReturn=\"y\" isStart=\"y\" />" +
	    "</class>" +
	    "<class name=\"E\" >" +
	    "</class>" +
	    "<class name=\"F\" >" +
	    "</class>" +
	    "</project>" ;


	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testReturnTag

    public void testSameObject() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] { "isReturn", "isStart", "name", 
			      "kind", "target", "incomingOrder", "method" } ) ;
	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"mE1\" this=\"a\" />" +
	    "    <call entity=\"B\" method=\"mB2\" this=\"a\" />" +
	    "    <call entity=\"D\" method=\"mD1\" this=\"b\" />" +
	    "    <call entity=\"B\" method=\"mB1\" this=\"a\" />" +
	    "  </trace> " +
	    "</traceSet>" ;
	
	String expected =
	    "<project>" +
	    "<class name=\"B\" isRoot=\"y\" >" +
	    "    <link callOrder=\"1\" method=\"mD1\" name=\"mD1\" kind=\"call\" "+
	    "           precedent=\"a\" target=\"D\" isReturn=\"y\" isStart=\"y\" />" +
	    "    <link callOrder=\"2\" method=\"mE1\" name=\"mE1\" kind=\"call\" "+
	    "           precedent=\"b\" target=\"E\" isStart=\"y\" isReturn=\"y\" "+
	    "          isSameObject=\"y\" "+
	    "           isLeaf=\"y\" />" +
	    "</class>" +
	    "<class name=\"D\" >" +
	    "    <link method=\"mB2\" name=\"mB2\" kind=\"call\" target=\"B\" "+
	    "           incomingOrder=\"b\" isReturn=\"y\" isStart=\"y\" />" +
	    "</class>" +
	    "<class name=\"E\" >" +
	    "</class>" +
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testSameObject
    
    public void testMultipleCallOrderObject() throws Exception {

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{   "name",
 		    "callOrder",
 		    "incomingOrder",
 		    "precedent"
		} ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"c\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"D\" method=\"d\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"c\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "</traceSet>" ;
	
	String expected =
	    "<project>" +
	    "  <class name=\"A\" >" +
	    "    <link name=\"b\" target=\"B\" callOrder=\"1-2\" incomingOrder=\"a-b\" "+
	    "          precedent=\"a-b\" />" +
	    "  </class>" +
	    "  <class name=\"B\" >" +
	    "    <link name=\"c\" target=\"C\" callOrder=\"1-2\" incomingOrder=\"a-b\" "+
	    "          precedent=\"a-b\" />" +
	    "  </class>" +
	    "  <class name=\"C\" >" +
	    "  </class>" +
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

	input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"c\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"D\" method=\"d\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"D\" method=\"d\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"d\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"e\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "</traceSet>" ;
	
	expected =
	    "<project>" +
	    "  <class name=\"A\" >" +
	    "    <link name=\"b\" target=\"B\" callOrder=\"1-3\" incomingOrder=\"a-c\" "+
	    "          precedent=\"a-c\" />" +
	    "  </class>" +
	    "  <class name=\"B\" >" +
	    "    <link name=\"c\" target=\"C\" callOrder=\"1\" "+
	    "          precedent=\"a\" />" +
	    "    <link name=\"d\" target=\"D\" callOrder=\"2\" "+
	    "          precedent=\"b\" />" +
	    "    <link name=\"e\" target=\"E\" callOrder=\"3\" "+
	    "          precedent=\"c\" />" +
	    "  </class>" +
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testMultipleCallOrderObject

    public void testMultipleCallOrderObjectExplicitRange() throws Exception {
    
	CallGraphToCondensedGraphTransformer.setExplicitRange(true) ;

	PropertyBag.DebugSetPropertiesRelevantForEquality
	    (  new String[] 
		{   "name",
 		    "callOrder",
 		    "incomingOrder",
 		    "precedent"
		} ) ;

	String input =
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"c\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"D\" method=\"d\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"D\" method=\"d\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"d\" />" +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"E\" method=\"e\" />" +
	    "    <call entity=\"B\" method=\"b\" />" +
	    "    <call entity=\"A\" method=\"a\" />" +
	    "  </trace> " +
	    "</traceSet>" ;
	
	String expected =
	    "<project>" +
	    "  <class name=\"A\" >" +
	    "    <link name=\"b\" target=\"B\" callOrder=\"1,2,3\" incomingOrder=\"a,b,c\" "+
	    "          precedent=\"a,b,c\" />" +
	    "  </class>" +
	    "  <class name=\"B\" >" +
	    "    <link name=\"c\" target=\"C\" callOrder=\"1\" "+
	    "          precedent=\"a\" />" +
	    "    <link name=\"d\" target=\"D\" callOrder=\"2\" "+
	    "          precedent=\"b\" />" +
	    "    <link name=\"e\" target=\"E\" callOrder=\"3\" "+
	    "          precedent=\"c\" />" +
	    "  </class>" +
	    "</project>" ;

	testEqualityAfterTransformFromTrace(input,expected) ;

    } // EndMethod testMultipleCallOrderObjectExplicitRange

    
    /** ****************************************************
     * Test a graph whose conversion does not work
     * <p><b>Date:</b> Sat Aug 25 2007 10:35:47
     * @author Francois Taiani
     */
    public void testProblematicGraph1() throws Exception {
	String problemGraph = 
	    "<project " +
	    "type=\"callGraph\" >" +
	    "<class eventOrder=\"1\" entity=\"B\" name=\"B::mB1'1\" kind=\"call\" method=\"mB1\" >" +
	    "    <link subkind=\"call\" name=\"C::mC1'2\" kind=\"controlflow\" target=\"C::mC1'2\" />" +
	    "</class>" +
	    "<class eventOrder=\"2\" entity=\"C\" name=\"C::mC1'2\" kind=\"call\" method=\"mC1\" >" +
	    "</class>" +
	    "<class eventOrder=\"3\" entity=\"C\" name=\"C::mC2'3\" kind=\"call\" method=\"mC2\" >" +
	    "    <link subkind=\"call\" name=\"B::mB2'4\" kind=\"controlflow\" target=\"B::mB2'4\" />" +
	    "</class>" +
	    "<class eventOrder=\"4\" entity=\"B\" name=\"B::mB2'4\" kind=\"call\" method=\"mB2\" >" +
	    "</class>" +
	    "</project>" ;

	String expected =
	    "<project>" +
	    "  <class name=\"B\" isRoot=\"y\" >" +
	    "    <link name=\"mC1\" target=\"C\" "+
	    "          precedent=\"a\" "+
	    "          method=\"mC1\" "+
	    "          isStart=\"y\" "+
	    "          incomingOrder=\"a\" "+
	    "          isLeaf=\"y\" "+
	    "          kind=\"call\" "+
	    "          isReturn=\"y\" "+
	    "          globalOrder=\"1\" />" +
	    "  </class>" +
	    "  <class name=\"C\" isRoot=\"y\" >" +
	    "    <link name=\"mB2\" target=\"B\" "+
	    "          precedent=\"b\" "+
	    "          method=\"mB2\" "+
	    "          isStart=\"y\" "+
	    "          incomingOrder=\"b\" "+
	    "          isLeaf=\"y\" "+
	    "          kind=\"call\" "+
	    "          isReturn=\"y\" "+
	    "          globalOrder=\"2\" />" +
	    "  </class>" +
	    "</project>" ;

	testEqualityAfterTransformFromString(problemGraph,expected) ;

	String problemGraphAsTrace = 
	    "<traceSet>" +
	    "  <trace> " +
	    "    <call entity=\"C\" method=\"mC1\" />" +
	    "    <call entity=\"B\" method=\"mB1\" />" +
	    "    <call entity=\"A\" method=\"mA1\" /> " +
	    "  </trace> " +
	    "  <trace> " +
	    "    <call entity=\"B\" method=\"mB2\" />" +
	    "    <call entity=\"C\" method=\"mC2\" />" +
	    "    <call entity=\"A\" method=\"mA1\" /> " +
	    "  </trace> " +
	    "</traceSet>" ;
	
 	CallGraph testee
	    = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(problemGraphAsTrace);

	performEdgeIteratorComparisonAfterRemoval(testee,expected);

	performEdgeIteratorComparisonAfterRemoval((CallGraph)testee.clone(),expected);

	// Now the last test: we close the testee before doing the test.

    } // EndMethod testProblematicGraph1

    /** ****************************************************
     * Used in testProblematicGraph1
     * <p><b>Date:</b> Sat Aug 25 2007 11:52:33
     * @author Francois Taiani
     */
    private void performEdgeIteratorComparisonAfterRemoval(CallGraph testee,
							   String    expected )
	throws Exception
    {
	// We condense a first time to exercise the condenser
	DirectedGraphWithDict
	    condensedTestee = condenserTestee.fromCallGraphToGraph(testee);

	java.util.Iterator allANodesIt =  testee.getNodeSWithRegExp("A*").iterator() ;

	while(allANodesIt.hasNext()) 
	    testee.remove( (NamedVertex)allANodesIt.next() ) ;

	testEqualityAfterTransformFromGraph(testee,expected);

	// Testing that edge iterators work properly

	DirectedGraphWithDict expectedTransformGraph = graphReader.readGraphFromXML
	    ( new ByteArrayInputStream(expected.getBytes()) ) ;

	condensedTestee = condenserTestee.fromCallGraphToGraph(testee);

	assertEquals(constructEdgeSetFromIterator(expectedTransformGraph),
		     constructEdgeSetFromIterator(condensedTestee));
    } // EndMethod performEdgeIteratorComparisonAfterRemoval

    private Set constructEdgeSetFromIterator(DirectedGraphWithDict aGraph) {
	Iterator iterOnEdges = aGraph.getEdgeIterator() ;
	Set result = new TreeSet();
	while(iterOnEdges.hasNext()) {
	    result.add(iterOnEdges.next());
	} // EndWhile
	return result ;
    } // EndMethod constructEdgeSetFromIterator

    static public void main(String argc[]) throws Exception {
	new TestCallGraphToCondensedGraphTransformer("").testMultipleCallOrderObject() ;
    } // EndMain

} // EndClass TestCallGraphToCondensedGraphTransformer
