/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestDynamicEdgeRendering.java
 * 
 * Time-stamp: <2002-08-16 22:32:53 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestDynamicEdgeRendering.java,v 1.4 2004/09/09 15:31:01 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.PropertyBag ;

import java.io.ByteArrayOutputStream ;
import java.io.ByteArrayInputStream ;
/**
 * Tests the dynamic rendering facility for edges
 * in {@link DirectedGraphWithDict2DotOStream}.
 * @see PropertyRenderer
 * @see DirectedGraphWithDict2DotOStream#registerRendererForKindOfLink(PropertyRenderer,String)
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.4 $
 * @since Version 0.1.0 [under work]
 */
public class TestDynamicEdgeRendering
extends BagBasedTest {

    public TestDynamicEdgeRendering(String name) {
	super(name) ;
    } // EndConstructor TestDynamicEdgeRendering

    protected class AShouldNotBeCalledRenderer 
	implements	PropertyRenderer {
	public String formatProperty( String      kindOfLink,
				      PropertyBag theLinkProperties,
				      NamedEdge    linkToBeRendered,
				      DirectedGraphWithDict contextGraph
				      ) {
	    fail("That renderer should not be called. Kind "+
		 kindOfLink + " not present.") ;
	    return null ;
	}
    } // EndClass AShouldNotBeCalledRenderer

    protected class ARendererWeReWaitngFor
	implements	PropertyRenderer {
	public String formatProperty( String      kindOfLink,
				      PropertyBag theLinkProperties,
				      NamedEdge    linkToBeRendered,
				      DirectedGraphWithDict contextGraph
				      ) {
	    return "VALUE = "+theLinkProperties.getValueForProperty("myProperty") ;
	}
    } // EndClass ARendererWeReWaitngFor

    public void testDynamicEdgeRendering() 
	throws Exception {
	String graph = 
	    "<project>" +
	    "  <class name=\"A\"> " +
	    "    <link target = \"B\" kind        = \"myKind\"   "
	    +                        "myProperty  = \"Hoola\"    " 
	    +                        "myProperty2 = \"Hoola2\" />" +
	    "  </class> " + 
	    "  <class name=\"B\"> " +
	    "  </class> " + 
	    "</project>" ;
	
	XMLGraphReader readerTestee = 
	    new XMLGraphReader();

	DirectedGraphWithDict graphTestee = 
	    readerTestee.readGraphFromXML( new ByteArrayInputStream(graph.getBytes()) ) ;
	ByteArrayOutputStream tempByteArray = 
	    new ByteArrayOutputStream() ;

	DirectedGraphWithDict2DotOStream
	    writer = new DirectedGraphWithDict2DotOStream(tempByteArray);
	writer.registerRendererForKindOfLink
	    (new AShouldNotBeCalledRenderer(),"nonExistingKind") ;
	writer.registerRendererForKindOfLink
	    (new ARendererWeReWaitngFor(),"myKind") ;
	writer.writeGraphInDotFormat(graphTestee) ;
	String resultingString = tempByteArray.toString() ;
	tempByteArray.close() ;
	checkFormattedStringIsInGraph("VALUE = "+"Hoola" ,resultingString) ;
    } // EndMethod testDynamicEdgeRendering

    /** 
     * Used in {@link #testDynamicEdgeRendering()} 
     */
    private void checkFormattedStringIsInGraph(String format, String graph) {
	int indexOfFormat = graph.indexOf(format);
	assertTrue("The format of property myProperty does not appear in dot output",
	       indexOfFormat!=-1 );
	int endOfFormat = indexOfFormat + format.length() ;
	assertTrue("The format string is not preceeded by ' ' or '['",
	       ( graph.charAt(indexOfFormat-1)==' ' ||
		 graph.charAt(indexOfFormat-1)=='[' 
		 ) 
	       ) ; // End assert
	assertTrue("The format string is not followed by ' ' or ']'",
	       ( graph.charAt(endOfFormat)==' ' ||
		 graph.charAt(endOfFormat)==']' 
		 ) 
	       ) ; // End assert
    } // EndMethod checkFormattedStringIsInGraph

} // EndClass TestDynamicEdgeRendering
