/*
 * Copyright 2004 Francois Taiani <f.taiani@computer.org>
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestOnTheFlyCallGraphBuilder.java
 * 
 * Time-stamp: <>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: TestOnTheFlyCallGraphBuilder.java,v 1.3 2004/09/10 16:30:49 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.graphAnalysis.test ;

import laas.friends.cosmopen.graphAnalysis.* ;

/** ***************************************************************
 * @author Francois Taiani   <f.taiani@computer.org>
 */
public class TestOnTheFlyCallGraphBuilder extends BagBasedTest {

    public TestOnTheFlyCallGraphBuilder (String name) {
	super(name) ;
    } // EndConstructor TestOnTheFlyCallGraphBuilder

    public void testAddCallOnTheFly() throws Exception {
 	String expectedGraphString = 
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"D\" method=\"callOnD\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\"> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\"> " +
	    "    <call entity=\"C\" method=\"callOnC\" />" +
	    "    <call entity=\"B\" method=\"callOnB\" />" +
	    "    <call entity=\"A\" method=\"callOnA\" />" + 
	    "  </trace> " +
	    "</traceSet>" ;

        CallGraph expected ;
        CallGraph result ;

 	expected = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(expectedGraphString) ;

        OnTheFlyCallGraphBuilder testee = new OnTheFlyCallGraphBuilder() ;

        testee.addCallOnTheFly("thread 1",1,"A","callOnA",null) ;
        testee.addCallOnTheFly("thread 1",2,"B","callOnB",null) ;

        testee.addCallOnTheFly("thread 2",1,"A","callOnA",null) ;
        testee.addCallOnTheFly("thread 2",2,"B","callOnB",null) ;

        testee.addCallOnTheFly("thread 1",1,"D","callOnD",null) ;
        testee.addCallOnTheFly("thread 1",2,"B","callOnB",null) ;
        testee.addCallOnTheFly("thread 1",3,"C","callOnC",null) ;

        testee.addCallOnTheFly("thread 1",1,"A","callOnA",null) ;
        testee.addCallOnTheFly("thread 1",2,"B","callOnB",null) ;
        testee.addCallOnTheFly("thread 1",3,"C","callOnC",null) ;

        testee.addCallOnTheFly("thread 2",3,"C","callOnC",null) ;

        result = testee.endGraphConstructionAndReturnResult() ;

        assertEquals("Assert 1",expected,result);

    } // EndMethod testAddCallOnTheFly

    public void testAddCallOnTheFlyWithThreadCreation() throws Exception {

        String expectedGraphString = 
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"2\" />" +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"2\" >  " +
	    "    <call entity=\"F\" method=\"\" />" +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"D\" method=\"\" />" +
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"G\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expected ;
        CallGraph result ;

 	expected = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(expectedGraphString) ;

        OnTheFlyCallGraphBuilder testee = new OnTheFlyCallGraphBuilder() ;
        
        testee.addCallOnTheFly("thread 1",1,"A","",null) ;
        testee.addCallOnTheFly("thread 1",2,"B","",null) ;
        testee.addCallOnTheFly("thread 1",2,"C","",null) ;

        testee.registerNewThread("thread 2") ;

        testee.addCallOnTheFly("thread 2",1,"D","",null) ;
        testee.addCallOnTheFly("thread 2",2,"B","",null) ;
        testee.addCallOnTheFly("thread 2",3,"F","",null) ;

        testee.addCallOnTheFly("thread 1",2,"G","",null) ;

        result = testee.endGraphConstructionAndReturnResult() ;

        assertEquals("Assert 2",expected,result);

    } // EndMethod testAddCallOnTheFlyWithThreadCreation

    /**
     * @see
     * TestXMLTraceSet2CallGraphReader#testCallGraphConstructionWithDelayedThreadCreation()
     */
    public void testAddCallOnTheFlyWithDelayedThreadCreation() throws Exception {

        String expectedGraphString = 
            "<traceSet threadCreation=\"B::clone\" >" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"clone\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"2\" > " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "    <call entity=\"C\" method=\"\" />" + 
	    "  </trace> " +
	    "  <new thread=\"3\" />" +
	    "  <trace thread=\"3\" > " +
	    "    <call entity=\"E\" method=\"\" />" +
	    "  </trace> " +
	    "</traceSet>" ;

        CallGraph expected ;
        CallGraph result ;

 	expected = XMLTraceSet2CallGraphReader.ReadCallGraphFromString(expectedGraphString) ;

        OnTheFlyCallGraphBuilder testee = new OnTheFlyCallGraphBuilder() ;
        
        testee.setUsualThreadCreationPoint("B::clone");

        testee.addCallOnTheFly("thread 1",1,"A",""     ,null) ;
        testee.addCallOnTheFly("thread 1",2,"B","clone",null) ;

        testee.addCallOnTheFly("thread 2",1,"C",""     ,null) ;
        testee.addCallOnTheFly("thread 2",2,"D",""     ,null) ;

        testee.registerNewThread("thread 3") ;

        testee.addCallOnTheFly("thread 3",1,"E",""    ,null) ;

        result = testee.endGraphConstructionAndReturnResult() ;

        assertEquals("Assert 3",expected,result);

    } // EndMethod testAddCallOnTheFlyWithDelayedThreadCreation

    public void testAddCallOnTheFlyWithObjectCreation() throws Exception {
	String expectedResult = 

	    "<project type=\"callGraph\" >" +
	    "  <class name=\"A::'t1-0\" entity=\"A\" " +
            "         method=\"\" eventOrder=\"0\" threadID=\"1\" > " +
	    "    <link target = \"B::&lt;init>'t1-1\" kind=\"controlflow\" "+
            "          subkind=\"create\" />" +
	    "  </class> " + 
	    "  <class name=\"B::&lt;init>'t1-1\" entity=\"B\" " +
            "         method=\"&lt;init>\" eventOrder=\"1\" "+
	    "         threadID=\"1\" kind=\"create\" > " +
	    "  </class> " + 
	    "</project>" ;
        
	XMLCallGraphReader graphReader = new XMLCallGraphReader();

	CallGraph expected ;
        CallGraph result ;

	expected = (CallGraph)graphReader.readGraphFromXML
	    ( new java.io.ByteArrayInputStream(expectedResult.getBytes()) ) ;
        
// 	    <traceSet>
// 	      <trace thread="1" > 
// 	        <call entity="B" method="<init>" />
// 	        <call entity="A" method="" /> 
// 	      </trace> 
// 	    </traceSet>

        OnTheFlyCallGraphBuilder testee = new OnTheFlyCallGraphBuilder() ;
        
        testee.addCallOnTheFly("thread 1",1,"A",""     ,null) ;
        testee.addCallOnTheFly("thread 1",2,"B","<init>",null) ;

        result = testee.endGraphConstructionAndReturnResult() ;

        assertEquals("Assert 4",expected,result);

    } // EndMethod testAddCallOnTheFlyWithObjectCreation

    public void testAddThrownException() throws Exception {

        String expectedGraphString = 
	    "<traceSet>" +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"B\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
            "  </trace> " +
	    "  <trace thread=\"1\" > " +
            "    <call entity=\"AnException\" method=\"throw\" />" +
	    "    <call entity=\"C\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "  <trace thread=\"1\" > " +
	    "    <call entity=\"D\" method=\"\" />" +
	    "    <call entity=\"A\" method=\"\" />" + 
	    "  </trace> " +
	    "</traceSet>" ; 

        CallGraph expected ;
        CallGraph result ;

 	expected = 
            XMLTraceSet2CallGraphReader
            .ReadCallGraphFromString(expectedGraphString) ;

        OnTheFlyCallGraphBuilder testee = new OnTheFlyCallGraphBuilder() ;
        
        testee.addCallOnTheFly("thread 1",1,"A","",null) ;
        testee.addCallOnTheFly("thread 1",2,"B","",null) ;
        testee.addCallOnTheFly("thread 1",2,"C","",null) ;
        testee.addThrownException("thread 1","AnException") ;
        testee.addCallOnTheFly("thread 1",2,"D","",null) ;
       
        result = testee.endGraphConstructionAndReturnResult() ;

        assertEquals("Assert 5",expected,result);
        

    } // EndMethod testAddThrownException
    
} // EndClass TestOnTheFlyCallGraphBuilder
