/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestSmallClasses.java
 * 
 * Time-stamp: <2003-02-24 18:04:27 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestSmallClasses.java,v 1.6 2004/10/27 17:16:53 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;
import laas.friends.cosmopen.utils.PropertyBag ;

/**
 * A class to test small classes in Hydrogen2Dot.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.6 $
 * @since Version 0.0.1
 */
public class TestSmallClasses
extends BagBasedTest {

    public TestSmallClasses(String name) {
	super(name) ;
    } // EndConstructor TestSmallClasses

    /**
     * tests class {@link PropertyBag}
     */
    public void testPropertyBag() {
	PropertyBag propertyBagTestee ;
	propertyBagTestee = new PropertyBag() ;
	propertyBagTestee.setPropertyToValue("NAME","Testee");
	assertTrue( propertyBagTestee.hasProperty("NAME")) ;
	assertTrue(!propertyBagTestee.hasProperty("NaMe")) ;
	assertTrue(!propertyBagTestee.hasProperty(""    )) ;
	assertEquals
	    ( propertyBagTestee.getValueForProperty("NAME"),
	      "Testee" 
	      ) ; // EndCall assertEquals
	assertEquals
	    ( propertyBagTestee.getValueForProperty("NaMe"),
	      null 
	      ) ; // EndCall assertEquals

	// TESTS EQUALS METHOD

	PropertyBag secondTestee = new PropertyBag() ;
	secondTestee.setPropertyToValue("NICKNAME","Testee2");
	assertTrue(!secondTestee.equals(propertyBagTestee) ) ;
	secondTestee.setPropertyToValue("NAME","Testee");
	assertTrue(!secondTestee.equals(propertyBagTestee) ) ;
	propertyBagTestee.setPropertyToValue("NICKNAME","Testee2");
	assertEquals(secondTestee,propertyBagTestee ) ;
	
    } // EndMethod testPropertyBag

    public void testCompareNamedVertex() {
	NamedVertex vertex1 = new NamedVertex("abcd") ;
	NamedVertex vertex2 = new NamedVertex("abcdef") ;

	assertTrue("vertex1.compareTo(vertex1)",vertex1.compareTo(vertex1)==0);
	assertTrue("vertex2.compareTo(vertex2)",vertex2.compareTo(vertex2)==0);
	assertTrue("vertex1.compareTo(vertex2)",vertex1.compareTo(vertex2)<0);
	assertTrue("vertex2.compareTo(vertex1)",vertex2.compareTo(vertex1)>0);

	vertex2 = new NamedVertex("abcdef") ;
	vertex2.setPropertyToValue("eventOrder","9");
	assertTrue("vertex2[eo].compareTo(vertex2[eo])",vertex2.compareTo(vertex2)==0);
	assertTrue("vertex1.compareTo(vertex2[eo])",vertex1.compareTo(vertex2)<0);
	assertTrue("vertex2[eo].compareTo(vertex1)",vertex2.compareTo(vertex1)>0);

	vertex1 = new NamedVertex("abcd") ;
	vertex1.setPropertyToValue("eventOrder","20");
	assertTrue("vertex1[eo].compareTo(vertex2[eo])",vertex1.compareTo(vertex2)>0);
	assertTrue("vertex2[eo].compareTo(vertex1[eo])",vertex2.compareTo(vertex1)<0);

	vertex2 = new NamedVertex("abcdef") ;
	vertex2.setPropertyToValue("eventOrder","100");
	assertTrue("vertex1[eo].compareTo(vertex2[eo2])",vertex1.compareTo(vertex2)<0);
	assertTrue("vertex2[eo2].compareTo(vertex1[eo])",vertex2.compareTo(vertex1)>0);

    } // EndMethod testCompareNamedVertex

} // EndClass TestSmallClasses
