/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * TestXMLGraphWriter.java
 * 
 * Time-stamp: <02/08/16 11:49:42 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: TestXMLGraphWriter.java,v 1.1 2004/09/09 15:35:38 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.graphAnalysis.test  ;

import laas.friends.cosmopen.graphAnalysis.* ;

import java.io.ByteArrayInputStream    ;
import java.io.File ;

/**
 * Test Class for {@link XMLGraphWriter }
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.1 $
 * @since Version 0.0.4
 */
public class TestXMLGraphWriter 
    extends BagBasedTest {

    public TestXMLGraphWriter(String name) {
	super(name) ;
    } // EndConstructor TestXMLGraphWriter

    /****************************************************************
     * create an exemplary graph to be used in some test methods.
     */
    private DirectedGraphWithDict createGraphTestee() 
	throws Exception 
    {
	XMLGraphReader xmlReader ;
	DirectedGraphWithDict     graphTestee ;
	String                    inputString ;

	inputString = 
	    "<project>" +
	    "<class name=\"forward_print_impl\"  >" +
	    "    <link kind=\"superclass\" target=\"print_impl\" />" +
	    "    <link kind=\"usage\" name=\"pobj\" " +
	    "          target=\"print_service\" referenceDepth=\"0\" "+
	    "          template=\"OB::ObjVar&lt;print_service>\" />" +
	    "</class>" +
	    "<class name=\"OBDirectStubImpl_print_service\"  >" +
	    "    <link kind=\"superclass\" target=\"OBStubImpl_print_service\" />" +
	    "    <link kind=\"superclass\" target=\"OB::DirectStubImpl\" />" +
	    "</class>" +
	    "<class name=\"OBMarshalStubImpl_print_service\"  >" +
	    "    <link kind=\"superclass\" target=\"OBStubImpl_print_service\" />" +
	    "    <link kind=\"superclass\" target=\"OB::MarshalStubImpl\" />" +
	    "</class>" +
	    "<class name=\"OBStubImpl_print_service\"  >" +
	    "    <link kind=\"superclass\" target=\"OB::StubImplBase\" />" +
	    "</class>" +
	    "<class name=\"Options\"  >" +
	    "</class>" +
	    "<class name=\"POA_print_service\"  >" +
	    "    <link kind=\"superclass\" " +
	    "          target=\"PortableServer::ServantBase\" />" +
	    "</class>" +
	    "<class name=\"print_impl\"  >" +
	    "    <link kind=\"superclass\" target=\"POA_print_service\" />" +
	    "</class>" +
	    "<class name=\"print_service\"  >" +
	    "    <link kind=\"superclass\" target=\"CORBA::Object\" />" +
	    "</class>" +
	    "</project>" ;

	xmlReader = new XMLGraphReader();

	graphTestee = xmlReader.readGraphFromXML( new ByteArrayInputStream(inputString.getBytes())) ;

	return graphTestee ;
	
    } // EndMethod createGraphTestee

    public void testWriteGraphInXML() throws Exception {
	DirectedGraphWithDict testee = createGraphTestee() ;
	File temporaryDotFile = File.createTempFile("opsbrowser", "tmp") ;

	java.io.OutputStream output 
	    = new java.io.FileOutputStream(temporaryDotFile) ;
	XMLGraphWriter
	    writer = new XMLGraphWriter(output);
	writer.writeGraph(testee) ;

	output.close() ;

	java.io.InputStream input = 
	    new java.io.FileInputStream (temporaryDotFile) ;

	XMLGraphReader 
	    reader = new XMLGraphReader() ; 
	DirectedGraphWithDict 
	    readGraph = reader.readGraphFromXML( input ) ;
	assertEquals(testee,readGraph) ;
	temporaryDotFile.delete() ;

    } // EndMethod testWriteGraphInXML

} // EndClass TestXMLGraphWriter
