/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * GroupOfDocumentedItemS.java
 * 
 * Time-stamp: <2003-02-16 18:36:52 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * $Id: GroupOfDocumentedItemS.java,v 1.2 2003/04/07 18:10:37 ftaiani Exp $
 * 
 */

package laas.friends.cosmopen.interpreter;

import java.util.* ;
import laas.friends.cosmopen.utils.MiscUtils ;

/**
 * This class groups together several {@link Command}s instances
 * that have some common semantic. The grouping is for documentation
 * and help purposes. The aim is to provide a hierachical help
 * similar to the one of <code>gdb</code> (the GNU debugger).
 * Actually a <code>GroupOfDocumentedItemS</code> may also itself
 * contain other <code>GroupOfDocumentedItemS</code> besides plain {@link Command}s.
 * This is achieved by using the class {@link AbstractDocumentedItem}.
 * @author Francois Taiani   <taiani@laas.fr>
 */
public class GroupOfDocumentedItemS extends AbstractDocumentedItem {

    
    private SortedSet _groupContent = new TreeSet() ;

    /** **************************************************** */
    public GroupOfDocumentedItemS(String aCommandName) 
    {
	super(aCommandName) ;
    } // EndConstructor GroupOfDocumentedItemS

    /** **************************************************** */
    public GroupOfDocumentedItemS(String aCommandName,
				  String shortDescription,
				  String  longDescription ) 
    {
	super(aCommandName) ;
	this.setShortDescription ( shortDescription ) ;
	this.setLongDescription  (  longDescription ) ;
    } // EndConstructor GroupOfDocumentedItemS


    /** ****************************************************
     * adds a new documented item to the group. This item
     * is normally either a {@link Command} or a {@link GroupOfDocumentedItemS}.
     * This method is called directly by the {@link CommandInterpreter} when
     * a {@link Command} or a {@link GroupOfDocumentedItemS}  is added to the
     * interpreter and should not be called otherwise. It does not
     * make any checks of any kind.
     */
    public void addItemToGroup(AbstractDocumentedItem aDocumentedItem) {
	_groupContent.add(aDocumentedItem) ;
    } // EndMethod addItemToGroup


//     /** ****************************************************
//      * returns the documented item with the passed name if any.
//      */
//     public AbstractDocumentedItem getDocumentedItemWithName(String aName) {
// 	return null ;
//     }


    /** ****************************************************
     * returns a list of the contained documented item.
     */
    public SortedSet getGroupContent() {
	return _groupContent ;
    }

    /** ****************************************************
     * The help output of a {@link GroupOfDocumentedItemS} also returns
     * a list of the element of the group besides the text passed to
     * {@link AbstractDocumentedItem#setLongDescription(String)}.
     */
    public String getHelpOutput() {
	StringBuffer result = new StringBuffer(getName());
	result.append("\n\n") ;
	if (this.hasLongDescription()) {
	    result.append(this.getLongDescription()) ;
	    result.append("\n\n") ;
	} // EndIf this.hasLongDescription()
	for ( java.util.Iterator i = getGroupContent().iterator() ;
	      i.hasNext() ; ) 
	    {
		AbstractDocumentedItem currentItem = (AbstractDocumentedItem)i.next() ;
		result.append
		    (" "+currentItem.getBullet()+
		     " "+MiscUtils.ExtendWithSpaces (currentItem.getName(),9)+
		     " : "+ currentItem.getShortDescription() + "\n"
		     ) ;
	    } // EndFor Loop through commands
	result.append
	    ( "\nType 'help <item>' for a more "+
	      "detailed description of an item." ) ;
	return result.toString() ;
    } // EndMethod getLongDescription

    public char getBullet() {
	return '+' ;
    }
} // EndClass GroupOfDocumentedItemS
