/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * MiscUtils.java
 * 
 * Time-stamp: <2002-08-26 15:46:56 ftaiani>
 * @author Francois Taiani   <taiani@laas.fr>
 * CVS Info: $Id: MiscUtils.java,v 1.5 2004/10/04 21:47:43 ftaiani Exp $
 * 
 */
package laas.friends.cosmopen.utils  ;

/**
 * This class contains a set of miscelenous utilitary
 * methods.
 * @author Francois Taiani   <taiani@laas.fr>
 * @version $Name: release-1-3-rc1 $ $Revision: 1.5 $
 * @since Version 0.0.3
 */
public abstract class MiscUtils {

    /**
     * This methods adds spaces (' ') at the end of String
     * in order to reach a given length. If the passed String
     * is longuer than the passed sizeToReach, nothing happens,
     * the String is not changed. This method should be useful 
     * when printing column formatted output on a serial text
     * stream (like System.our).
     */
    public static String ExtendWithSpaces
	(String toBeExtended, int sizeToReach) {

	StringBuffer aux = new StringBuffer(toBeExtended) ;
	for( int i = aux.length() ; i < sizeToReach ; i++) {
	    aux.append(' ') ;
	} // EndFor
	return aux.toString() ;
    } // EndStatic ExtendWithSpaces

    static public void PutArrayInCollection
	( Object[]              args, 
	  java.util.Collection target ) 
    {
	for(int i=0; i<args.length; i++) {
	    target.add(args[i]) ;
	} // EndFor i
    } // EndMethod PutArrayInCollection

    /**
     * acts as a filter to replace a passed characters 
     * with a given substring.
     */
    public static String ReplaceCharWithString(String  toBeTransformed, 
					       char    oldChar, 
					       String  replacing ) 
    {

	StringBuffer tempBuff = new StringBuffer() ;
	for(int i=0; i < toBeTransformed.length() ; i++) {
	    if (toBeTransformed.charAt(i)==oldChar) {
		tempBuff.append(replacing) ;
// 		System.out.println("tempBuff:"+tempBuff) ;
	    }
	    else
		tempBuff.append(toBeTransformed.charAt(i)) ;
	    // [O] : This can clearly be optimized, but we don't care here.
	} // EndFor i
	return tempBuff.toString() ;

    } // EndMethod ReplaceCharWithString

    public static char charCoding(int anInt) {
	if (9+anInt>=Character.MAX_RADIX)
	    return  '*' ;
	else
	    return Character.forDigit(9+anInt,Character.MAX_RADIX) ;
    } // EndMethod charCoding

    /** **************************************************** 
     * Remove the package / namespace headers and the template
     * parameters from an entity name.  for instance
     * <code>MyNameSpace::MyFirstClass::MyNestedClass</code> becomes
     * <code>MyNestedClass</code>.
     */
    public static String GetShortEntityName(String aLongEntityName) {
	int justOnFirstGreaterThan=aLongEntityName.indexOf('<');
	if (justOnFirstGreaterThan==-1) // No greater than, this is
					// not a template
					// instanciation
	    justOnFirstGreaterThan=aLongEntityName.length() ;

	int justAfterLastColon=
	    aLongEntityName.lastIndexOf(':',justOnFirstGreaterThan)+1;

	int justAfterLastDot=
	    aLongEntityName.lastIndexOf('.',justOnFirstGreaterThan)+1;

        int justAfterLastDotOrColon = 
            (justAfterLastColon>justAfterLastDot?
             justAfterLastColon:justAfterLastDot) ;

	return
	    aLongEntityName
	    .substring(justAfterLastDotOrColon,
		       justOnFirstGreaterThan) ;
	
    } // EndMethod GetShortEntityName

    /**
     * acts as a filter to transform "<" characters in "\&lt;" and "&"
     * in "&amp;" in attribute values (required by XML)
     */
    public static String makeXMLCompliant(String toBeTransformed) {
	String result = toBeTransformed ;
	result = MiscUtils.ReplaceCharWithString(result,'&',"&amp;");
	result = MiscUtils.ReplaceCharWithString(result,'<',"&lt;");
	return result ;

    } // EndMethod transformLessThan

} // EndClass MiscUtils
