/*
 * Copyright 2003 LAAS-CNRS
 *
 *   LAAS-CNRS
 *   7, avenue du Colonel Roche
 *   F-31077 Toulouse Cedex 4 / France
 *
 * Author: Francois Taiani
 *
 * This file is part of OPSBrowser.
 * 
 * OPSBrowser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * OPSBrowser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with OPSBrowser; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
/**
 * main-observee.cpp
 * 
 * Francois Taiani   <taiani@laas.fr>
 * Time-stamp: <2003-04-07 17:18:38 ftaiani>
 * 
 */
#include <iostream.h>
#include <pthread.h>
#include <unistd.h>

class ClassC {

private:
  pthread_mutex_t _object_lock ;

public:

  ClassC() {
      pthread_mutex_init(&this->_object_lock,0) ;
  } // EndConstructor ClassC

  void printMessage() {
    pthread_mutex_lock(&this->_object_lock);
    cout << "This is thread n" << pthread_self() << endl ;
    pthread_mutex_unlock(&this->_object_lock);
  } // EndMethod printMessage

}; // EndClass ClassC

class ClassB {
private:
  ClassC& _classC ;
public:

  ClassB(ClassC& aClassC) : _classC(aClassC) {
  } ; // EndConstructor

  void startSecondThread() {
    _classC.printMessage() ;
  } // EndMethod startSecondThread

}; // EndClass ClassB

void * StartSecondThread(void * argument) {
  ClassB * aClassB = (ClassB *)argument ;
  aClassB->startSecondThread() ;
  return 0 ;
} // EndFunction StartSecondThread

class ClassA {

private:
  ClassC _classC ;
  ClassB _classB ;

public:
  ClassA(): _classC(), _classB(_classC) {} // EndConstructor

  void launch() {
    int result ;
    pthread_t secondThread ;
    result = pthread_create(&secondThread, NULL, 
			    StartSecondThread, (void *)&_classB) ;
    usleep(5000) ;
    _classC.printMessage();
  } // EndMethod launch

}; // EndClass ClassA

int main (int argc, char * argv[]) {
  ClassA mainClass ;
  mainClass.launch();
} // EndMain ;

